﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using MSIS.webservices.location;
using System.Resources;
using System.Globalization;

namespace MSIS
{
    public partial class LocationFinder : Form
    {        
        WiFi wifiHelper = new WiFi();
        LocationService locationService;
        Location[] matchingLocations = null; // Holds array of locations returned from search
        int selectedLocationIdx = -1; // Holds the index of the selected item in the location search results list
        MapSheet map;
        Location currentPos = null;
        TabPage currentTab = null;
        Dictionary<string, string> loctypes;

        #region LocationType inner class
        private class LocationType
        {
            private string displayName;
            private int value;
            public LocationType(String displayName, int value)
            {
                this.displayName = displayName;
                this.value = value;
            }
            public string DisplayName
            {
                get { return displayName; }
            }
            public int Value
            {
                get { return value; }
            }
        }
        #endregion

        public LocationFinder()
        {
            InitializeComponent();
            Text = AppSettings.AppTitle;

            try
            {
                locationService = new webservices.location.LocationService();
                locationService.Url = AppSettings.WebServiceUrl + "/" + AppSettings.LocationServiceName;
            }
            catch
            {
                Main.ShowConnectionError();
                Close();
            }
            finally
            {
                Cursor.Current = Cursors.Default;
            }

            radioWifiStatus.Text = Main.ResourceManager.GetString("mainWifiStatusRetrieving", AppSettings.Language);
            lblCtxSearchCurrentPos.Text = Main.ResourceManager.GetString("locfinCtxLblPosSearching", AppSettings.Language);

            map = new MapSheet(this);
            tabMap.Controls.Add(map);
            map.Dock = DockStyle.Top;
            
            // enable map description overlay
            //ToggleMapDescription(true);
            Cursor.Current = Cursors.Default;
            currentTab = tabSearch;

            SetLanguage();
        }

        public void LocationFinder_Load(object sender, EventArgs e)
        {
            tabControl.TabPages.Clear();
            tabControl.TabPages.Add(tabSearch);
            tabControl.SelectedIndex = 0;
            tabControl.SelectedIndexChanged += new EventHandler(tabControl_SelectedIndexChanged);
        }

        #region L10n content
        private void SetLanguage()
        {
            ResourceManager r = Main.ResourceManager;
            CultureInfo ci = AppSettings.Language;

            // Location types
            loctypes = new Dictionary<string, string>();
            string[] tmp = r.GetString("globalLocationTypes", ci).Split('|');

            foreach (string item in tmp)
            {
                loctypes.Add(item.Split('=')[0], item.Split('=')[1]);               
            }

            // Menu items
            menuItemMenu.Text = r.GetString("locfinMnu", ci);
            menuItemReturn.Text = r.GetString("locfinMnuReturn", ci);
            menuItemStdSearch.Text = r.GetString("locfinMnuStandardSearch", ci);
            menuItemWhere.Text = r.GetString("locfinMnuWhereAmI", ci);
            menuItemCtxSearch.Text = r.GetString("locfinMnuContextual", ci);
            menuItemDetails.Text = r.GetString("locfinCtxShowDetails", ci);
            menuItemShowMap.Text = r.GetString("locfinCtxShowMap", ci);

            // Form elements (Location Search)
            labelHeader.Text = r.GetString("locfinLblHeader", ci);
            labelKeyword.Text = r.GetString("locfinLblKeyword", ci);
            btnSearchLocations.Text = r.GetString("locfinBtnSearch", ci);
            linkReset.Text = r.GetString("locfinLnkReset", ci);
            linkShowContextual.Text = r.GetString("locfinTabContextual", ci);
            labelDescription.Text = r.GetString("locfinLblDescription", ci);

            listSearchResults.Columns[0].Text = r.GetString("locfinLstName", ci);
            listSearchResults.Columns[1].Text = r.GetString("locfinLstType", ci);
            listSearchResults.Columns[2].Text = r.GetString("locfinLstFloor", ci);

            // Form elements (Contextual Search)
            labelCtxHeader.Text = r.GetString("locfinCtxLblHeader", ci);
            labelCtxSearch.Text = r.GetString("locfinCtxLblSearch", ci);
            btnCtxSearch.Text = r.GetString("locfinCtxBtnSearch", ci);
            linkShowStandardSearch.Text = r.GetString("locfinCtxLnkStandard", ci);
            linkResetCtxSearch.Text = r.GetString("locfinLnkReset", ci);
            labelCtxResults.Text = r.GetString("locfinCtxLblNrOfResults", ci);
            radioCtxResultsAll.Text = r.GetString("locfinCtxRadioAllResults", ci);

            labelCtxPosDescription.Text = r.GetString("locfinCtxDescLocAware", ci);
            //
            labelCtxSearchDescription.Text = r.GetString("locfinCtxLblInstruct", ci);


            // Tabs
            tabControl.TabPages[0].Text = r.GetString("locfinTabLocationSearch", ci);
            tabControl.TabPages[1].Text = r.GetString("locfinTabContextual", ci);
            tabControl.TabPages[2].Text = r.GetString("locfinTabMap", ci);

            // Other
            lblNoWiFiCoverage.Text = r.GetString("locfinMsgNoLocationText", ci);


        }
        #endregion

        #region Handle tab selection changes
        private void ShowTab(object sender, EventArgs e)
        {
            Object item = sender;
            TabPage targetTab = null;
            int tabToRemoveIdx = 0;

            if (item == menuItemWhere)
            {
                Cursor.Current = Cursors.WaitCursor;
                currentPos = GetCurrentLocation();
               


                

                if (currentPos != null)
                {
                    map.ClearLocations();
                    if (!map.AddLocation(currentPos) || !map.SetSelectedLocationIndex(map.NumberOfLocations - 1))
                    {
                        map.ClearLocations();
                        MessageBox.Show(Main.ResourceManager.GetString("locfinMsgNoMapDataText", AppSettings.Language), Main.ResourceManager.GetString("locfinMsgNoMapDataTitle", AppSettings.Language),
                            MessageBoxButtons.OK, MessageBoxIcon.Exclamation, MessageBoxDefaultButton.Button1);
                        Cursor.Current = Cursors.Default;
                        return;
                    }
                    targetTab = tabMap;
                    Cursor.Current = Cursors.Default;
                }
                else
                {
                    DialogResult dr = MessageBox.Show(Main.ResourceManager.GetString("locfinMsgNoLocationText", AppSettings.Language),
                        Main.ResourceManager.GetString("locfinMsgNoLocationTitle", AppSettings.Language), MessageBoxButtons.RetryCancel, MessageBoxIcon.Exclamation, MessageBoxDefaultButton.Button1);
                    switch (dr)
                    {
                        case DialogResult.Cancel:
                            break;
                        case DialogResult.Retry:
                            ShowTab(item, new EventArgs());
                            break;
                    }
                }
                Cursor.Current = Cursors.Default;
            }

            else if (item == menuItemShowMap)
            {
                Cursor.Current = Cursors.WaitCursor;
                Location selLocation = matchingLocations[selectedLocationIdx];
                map.ClearLocations();
                if (!map.AddLocation(selLocation))
                {
                    MessageBox.Show(Main.ResourceManager.GetString("locfinMsgNoMapDataText", AppSettings.Language), Main.ResourceManager.GetString("locfinMsgNoMapDataTitle", AppSettings.Language),
                        MessageBoxButtons.OK, MessageBoxIcon.Asterisk, MessageBoxDefaultButton.Button1);
                    Cursor.Current = Cursors.Default;
                    map.ClearLocations();
                    return;
                }

                if (currentTab == tabSearchContextual)
                {
                    if (map.AddLocation(currentPos))
                    {
                        map.DrawLine(currentPos, selLocation);
                    }
                    else
                    {
                        MessageBox.Show(Main.ResourceManager.GetString("locfinMsgNoCoordinatesText", AppSettings.Language), Main.ResourceManager.GetString("locfinMsgNoCoordinatesTitle", AppSettings.Language),
                            MessageBoxButtons.OK, MessageBoxIcon.Exclamation, MessageBoxDefaultButton.Button1);
                    }
                }

                if (!map.SetSelectedLocationIndex(map.NumberOfLocations - 1))
                {
                    map.ClearLocations();
                    MessageBox.Show(Main.ResourceManager.GetString("locfinMsgNoMapDataText", AppSettings.Language), Main.ResourceManager.GetString("locfinMsgNoMapDataTitle", AppSettings.Language),
                        MessageBoxButtons.OK, MessageBoxIcon.Exclamation, MessageBoxDefaultButton.Button1);
                    Cursor.Current = Cursors.Default;
                    return;
                }

                targetTab = tabMap;
                Cursor.Current = Cursors.Default;
            }
            else if (item == menuItemStdSearch || item == linkShowStandardSearch)
            {
                linkResetSearchResults_Click(null, new EventArgs());
                if (tabControl.TabPages.Contains(tabMap))
                {
                    tabControl.TabPages.RemoveAt(tabControl.TabPages.IndexOf(tabMap));
                }
                targetTab = tabSearch;
                if (tabControl.TabPages.Contains(tabSearchContextual))
                {
                    tabToRemoveIdx = tabControl.TabPages.IndexOf(tabSearchContextual);
                    tabControl.TabPages.RemoveAt(tabToRemoveIdx);
                }
            }
            else if (item == menuItemCtxSearch || item == linkShowContextual)
            {
                linkResetSearchResults_Click(null, new EventArgs());
                if (tabControl.TabPages.Contains(tabMap))
                {
                    tabControl.TabPages.RemoveAt(tabControl.TabPages.IndexOf(tabMap));
                }
                targetTab = tabSearchContextual;
                if (tabControl.TabPages.Contains(tabSearch))
                {
                    tabToRemoveIdx = tabControl.TabPages.IndexOf(tabSearch);
                    tabControl.TabPages.RemoveAt(tabToRemoveIdx);
                }
            }

            // show the correct tab. If it is not in the list of controls, add it, otherwise give focus.
            if (targetTab != null)
            {
                if (!tabControl.TabPages.Contains(targetTab))
                {
                    tabControl.TabPages.Add(targetTab);
                }
                for (int i = 0; i < tabControl.TabPages.Count; i++)
                {
                    if (tabControl.TabPages[i] == targetTab)
                    {
                        tabControl.SelectedIndex = i;
                        return;
                    }
                }
            }
        }

        private void CloseTab(object sender, EventArgs e)
        {
            Control control = (sender as Control);
            TabPage parentTab = (TabPage)control.Parent;

            tabControl.SelectedIndex = 0;
            for (int i = 0; i < tabControl.TabPages.Count; i++)
            {
                if (tabControl.TabPages[i] == parentTab)
                {
                    tabControl.TabPages.RemoveAt(i);
                }
            }
        }

        private void tabControl_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (tabControl.TabPages.Count > 0)
            {
                currentTab = tabControl.TabPages[tabControl.SelectedIndex];

                // Include Wifi status indicator on tabpage
                if (currentTab == tabSearch || currentTab == tabSearchContextual)
                {
                    radioWifiStatus.Parent = currentTab;
                    Main.UpdateWifiIndicator(radioWifiStatus);
                }

                if (currentTab == tabMap)
                {
                    // show a information dialog if this is the first time the map is loaded
                    if (AppSettings.ShowHints)
                    {
                        MessageBox.Show(Main.ResourceManager.GetString("locfinMsgMapTipText", AppSettings.Language), Main.ResourceManager.GetString("locfinMsgMapTipTitle", AppSettings.Language), MessageBoxButtons.OK,
                            MessageBoxIcon.Asterisk, MessageBoxDefaultButton.Button1);
                    }
                }
                else if (currentTab == tabSearch)
                {
                    txtLocNameSearch.Text = "";
                }
                else if (currentTab == tabSearchContextual)
                {
                    // Reset screen
                    List<LocationType> lTypes = new List<LocationType>();
                    foreach (LocationTypes lt in Main.GetEnumValues(new LocationTypes()))
                    {
                        lTypes.Add(new LocationType(loctypes[lt.ToString()], (int)lt));
                    }
                    comboCtxSearch.DataSource = lTypes;
                    comboCtxSearch.DisplayMember = "DisplayName";
                    comboCtxSearch.ValueMember = "Value";
                    comboCtxSearch.SelectedIndex = -1;
                    radioCtxResults3.Checked = true;
                    lblCtxSearchCurrentPos.Text = Main.ResourceManager.GetString("locfinCtxLblPosSearching", AppSettings.Language);
                    ToggleCtxSearch(false);
                }
            }
        }
        #endregion

        #region Map descriptions
        /// <summary>
        /// Loads location information about given location into the map description overlay
        /// </summary>
        /// <param name="location">The location to show description for</param>
        /// <param name="showDescriptionOverlay">Whether or not the map description overlay should show</param>
        public void ShowMapDescription(Location location)
        {
            // update text on map description overlay
            lblMapDescriptionName.Text = location.Name + ((location.Floor > 0) ? " (" + Main.ResourceManager.GetString("locfinFloor", AppSettings.Language) + ": " + location.Floor + ")" : "");
            lblMapDescriptionDesc.Text = location.Description;
            ToggleMapDescription(true);
        }

        private void ToggleMapDescription(bool enable)
        {
            if (enable)
            {
                map.Size = new Size(map.Size.Width, tabMap.Bounds.Height - panelMapOverlay.Bounds.Height);
                lblMapDescriptionDesc.Size = new Size(lblMapDescriptionDesc.Size.Width, CalculateLabelHeight(lblMapDescriptionDesc));
                
                if (!tabMap.Controls.Contains(panelMapOverlay))
                {
                    tabMap.Controls.Add(panelMapOverlay);
                }
            }
            else
            {
                map.Size = new Size(Width, Height);
                if (tabMap.Controls.Contains(panelMapOverlay))
                {
                    tabMap.Controls.Remove(panelMapOverlay);
                }
            }
        }        
        
        private int CalculateLabelHeight(Label label)
        {
            float dpiFactor = AutoScaleDimensions.Height / 96.0F;
            Graphics g = this.CreateGraphics();
            SizeF size = g.MeasureString(label.Text, label.Font);
            int scaleY = (int)(size.Width / (label.Width - 5)) + 1;
            return Convert.ToInt32(scaleY * dpiFactor * 14);
        }
        #endregion

        #region Misc event handlers
        private void menuItemReturn_Click(object sender, EventArgs e)
        {
            Close();
        }
        
        private void txtLocNameSearch_GotFocus(object sender, EventArgs e)
        {
            // enable SIP
            inputPanel.Enabled = true;
        }

        private void tabMap_Resize(object sender, EventArgs e)
        {
            if (map != null)
            {
                map.Size = new Size(tabMap.Bounds.Width, tabMap.Bounds.Height - panelMapOverlay.Bounds.Height);
            }
        }

        #endregion

        #region Handle search form events (both contextual and standard)
        private void linkResetSearchResults_Click(object sender, EventArgs e)
        {
            listCtxSearchResults.Items.Clear();
            listSearchResults.Items.Clear();
            txtLocNameSearch.Text = "";
            listCtxSearchResults.Visible = false;
            comboCtxSearch.SelectedIndex = -1;
            radioCtxResults3.Checked = true;
            matchingLocations = null;
        }
        private void btnSearchLocations_Click(object sender, EventArgs e)
        {
            string searchString = txtLocNameSearch.Text;
            if (searchString.Length <= 0)
            {
                MessageBox.Show(Main.ResourceManager.GetString("locfinMsgKeywordText", AppSettings.Language), Main.ResourceManager.GetString("locfinMsgKeywordTitle", AppSettings.Language), MessageBoxButtons.OK, MessageBoxIcon.Asterisk, MessageBoxDefaultButton.Button1);
                return;
            }
            
            Cursor.Current = Cursors.WaitCursor;

            matchingLocations = null;
            try
            {
                matchingLocations = locationService.GetLocationByName(searchString);
            }
            catch (Exception)
            {
                Main.ShowConnectionError();
                Cursor.Current = Cursors.Default;
                return;
            }

            listSearchResults.Items.Clear();
            if (matchingLocations != null)
            {
                for (int i = 0; i < matchingLocations.Length; i++)
                {
                    Location location = matchingLocations[i];
                    ListViewItem item = new ListViewItem(location.Name);
                    item.SubItems.Add(loctypes[location.LocationType.ToString()]);
                    item.SubItems.Add(location.Floor.ToString());
                    listSearchResults.Items.Add(item);
                }
            }
            else
            {
                listSearchResults.Items.Add(new ListViewItem(Main.ResourceManager.GetString("locfinNoResults", AppSettings.Language)));
            }
            Cursor.Current = Cursors.Default;
            
        }

        private void btnCtxSearch_Click(object sender, EventArgs e)
        {
            if (comboCtxSearch.SelectedIndex == -1)
            {
                MessageBox.Show(Main.ResourceManager.GetString("locfinMsgSelectPointText", AppSettings.Language), Main.ResourceManager.GetString("locfinMsgSelectPointTitle", AppSettings.Language), MessageBoxButtons.OK, MessageBoxIcon.Asterisk, MessageBoxDefaultButton.Button1);
                return;
            }

            int numOfLocationsToReturn = 0;
            if (radioCtxResults1.Checked) { numOfLocationsToReturn = 1; }
            else if (radioCtxResults3.Checked) { numOfLocationsToReturn = 3; }
            else if (radioCtxResults5.Checked) { numOfLocationsToReturn = 5; }
            else if (radioCtxResultsAll.Checked) { numOfLocationsToReturn = -1; }

            if (wifiHelper.IsConnected)
            {
                Cursor.Current = Cursors.WaitCursor;
                matchingLocations = null;
                try
                {
                    matchingLocations = locationService.GetNearbyLocations(
                        (LocationTypes)comboCtxSearch.SelectedValue,
                        wifiHelper.signalVector,
                        numOfLocationsToReturn
                    );
                }
                catch (Exception ex)
                {
                    Main.ShowConnectionError();
                    Cursor.Current = Cursors.Default;
                    return;
                }

                listCtxSearchResults.Items.Clear();
                if (matchingLocations != null)
                {
                    for (int i = 0; i < matchingLocations.Length; i++)
                    {
                        Location location = matchingLocations[i];
                        ListViewItem item = new ListViewItem(location.Name);
                        if (location.DistanceFromCurrent != -1)
                        {
                            item.SubItems.Add(location.DistanceFromCurrent.ToString("~0 m"));
                        }
                        else
                        {
                            Main.ResourceManager.GetString("locfinDistanceUnknown", AppSettings.Language);
                        }
                        listCtxSearchResults.Items.Add(item);
                    }
                }
                else
                {
                    listCtxSearchResults.Items.Add(new ListViewItem(Main.ResourceManager.GetString("locfinNoResults", AppSettings.Language)));
                }
                listCtxSearchResults.Visible = true;
                listCtxSearchResults.BringToFront();

                Cursor.Current = Cursors.Default;
            }
            else
            {
                Main.ShowConnectionError();
            }
        }
        #endregion

        #region List context menu event handlers

        /// <summary>
        /// Shows a popup window with description of the location selected in the search results.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ShowDescriptionPopup(object sender, EventArgs e)
        {
            if (selectedLocationIdx >= 0)
            {
                Location selLocation = matchingLocations[selectedLocationIdx];

                string detailsText =
                    Main.ResourceManager.GetString("locfinPopName", AppSettings.Language) + ": " + selLocation.Name + Environment.NewLine +
                    Main.ResourceManager.GetString("locfinPopDesc", AppSettings.Language) + ": " + selLocation.Description + Environment.NewLine +
                    Main.ResourceManager.GetString("locfinPopFloor", AppSettings.Language) + ": " + selLocation.Floor + Environment.NewLine +
                    Main.ResourceManager.GetString("locfinPopType", AppSettings.Language) + ": " + loctypes[selLocation.LocationType.ToString()];
                PopUp detailsPopUp = new PopUp(detailsText);
                detailsPopUp.Title = Main.ResourceManager.GetString("locfinPopTitle");
                detailsPopUp.ShowDialog();
            }
        }

        /// <summary>
        /// Event handler for ListView ItemActivate event.
        /// Shows the context menu when a location is selected in the search results list.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ShowContextMenu(object sender, EventArgs e)
        {
            ListView lv = sender as ListView;
            if (lv.SelectedIndices.Count > 0 && matchingLocations != null)
            {
                selectedLocationIdx = lv.SelectedIndices[lv.SelectedIndices.Count - 1];
                /*
                foreach (ListViewItem item in lv.Items)
                {
                    if (item.Index != selectedLocationIdx)
                    {
                        item.Selected = false;
                    }
                }
                */
                lv.ContextMenu.Show(this, new Point(
                    lv.Location.X + 50, 30 + panelSearchResults.Location.Y + (selectedLocationIdx * 20)));
            }
            else
            {
                selectedLocationIdx = -1;
            }
        }
        #endregion

        #region Ticker event handler and helper methods that enables and disables features based on WiFi connectivity
        /**
        * Timer event handler
        * Continously polls for WiFi reception, and updates the frontend status.
        * 
        */
        private void timer_Tick(object sender, EventArgs e)
        {
            Main.UpdateWifiIndicator(radioWifiStatus);
            if (currentTab == tabSearchContextual)
            {
                UpdateCtxControls();
            }

            if (wifiHelper.IsConnected)
            {
                linkShowContextual.Enabled = true;
                menuItemCtxSearch.Enabled = true;
                menuItemWhere.Enabled = true;
            }
            else
            {
                linkShowContextual.Enabled = false;
                menuItemCtxSearch.Enabled = false;
                menuItemWhere.Enabled = false;
            }
        }

        private void ToggleCtxSearch(bool enable)
        {
            btnCtxSearch.Enabled = enable;
            comboCtxSearch.Enabled = enable;
            radioCtxResults1.Enabled = enable;
            radioCtxResults3.Enabled = enable;
            radioCtxResults5.Enabled = enable;
            radioCtxResultsAll.Enabled = enable;
        }


        private Location GetCurrentLocation()
        {
            Signal[] signals = null;
            try
            {
                signals = wifiHelper.signalVector;
            }
            catch (Exception e)
            {
                MessageBox.Show(Main.ResourceManager.GetString("locfinMsgNoSignalVector", AppSettings.Language));
                return null;
            }
            if (signals != null && wifiHelper.IsConnected)
            {
                try
                {
                    Location location = locationService.GetLocationByFingerprint(signals);
                    if (location != null)
                    {
                        return location;
                    }
                }
                catch (Exception e)
                {
                    // Could not retrieve location. Usually occurs only when the NIC is not completely connected to WiFi
                    return null;
                }
            }
            return null;
        }

        private void UpdateCtxControls()
        {
            currentPos = GetCurrentLocation();

            if (currentPos == null)
            {
                lblNoWiFiCoverage.Visible = true;
                panelCtxSearchCurrentPos.Visible = false;
                lblCtxSearchCurrentPos.Text = "";
                ToggleCtxSearch(false);
            }
            else
            {
                lblNoWiFiCoverage.Visible = false;
                panelCtxSearchCurrentPos.Visible = true;
                lblCtxSearchCurrentPos.Text = currentPos.Name + " (" + currentPos.Floor + ". " + Main.ResourceManager.GetString("locfinFloor") + ")";
                ToggleCtxSearch(true);
            }
        }

        #endregion

        private void listSearchResults_SelectedIndexChanged(object sender, EventArgs e)
        {

        }

        private void menuItemSearchUser_Click(object sender, EventArgs e)
        {
            new UserLocation().ShowDialog();
        }

        private void menuItemUsers_Click(object sender, EventArgs e)
        {
            new GetUsersForm().ShowDialog();
        }

        private void labelHeader_ParentChanged(object sender, EventArgs e)
        {

        }

    }
}

