﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Reflection;
using MSIS.webservices.location;
using System.Resources;
using System.Globalization;

namespace MSIS
{
    public partial class LocationManager : Form
    {
        private class LocationType
        {
            private string displayName;
            private int value;
            public LocationType(string displayName, int value)
            {
                this.displayName = displayName;
                this.value = value;
            }
            public string DisplayName
            {
                get { return displayName; }
            }
            public int Value
            {
                get { return value; }
            }
        }

        WiFi wifiHelper = new WiFi();
        LocationService locationService = null;
        Dictionary<string, string> locationTypes;

        public LocationManager()
        {
            InitializeComponent();
            SetLanguage();

            radioWifiStatus.Text = Main.ResourceManager.GetString("mainWifiStatusRetrieving", AppSettings.Language);

            // check if WIFI NIC exists, if not, close window
            if (!wifiHelper.IsConnected)
            {
                MessageBox.Show("No wireless connection found. This feature is not available.", "Not available",
                    MessageBoxButtons.OK, MessageBoxIcon.Exclamation, MessageBoxDefaultButton.Button1);
                Close();
            }

            try
            {
                locationService = new webservices.location.LocationService();
                locationService.Url = AppSettings.WebServiceUrl + "/" + AppSettings.LocationServiceName;
            }
            catch
            {
                Main.ShowConnectionError();
                Close();
            }
        }

        private void SetLanguage()
        {
            ResourceManager r = Main.ResourceManager;
            CultureInfo ci = AppSettings.Language;

            // L10N for combobox values
            locationTypes = new Dictionary<string, string>();
            string[] tmp = r.GetString("globalLocationTypes", ci).Split('|');

            foreach (string item in tmp)
            {
                locationTypes.Add(item.Split('=')[0], item.Split('=')[1]);
            }

            // Labels
            labelHeader.Text = r.GetString("locmanLblHeader", ci);
            labelDesc.Text = r.GetString("locmanLblDesc", ci);
            labelName.Text = r.GetString("locmanLblLocName", ci);
            labelType.Text = r.GetString("locmanLblType", ci);
            labelLocDescription.Text = r.GetString("locmanLblLocDesc", ci);
            
            // Menu items
            menuItemExit.Text = r.GetString("locmanMnuReturn", ci);
            menuItemSave.Text = r.GetString("locmanMnuSave", ci);

        }

        private void Configuration_Load(object sender, EventArgs e)
        {
            List<LocationType> lTypes = new List<LocationType>();

            foreach (LocationTypes lt in Main.GetEnumValues(new LocationTypes()))
            {
                lTypes.Add(new LocationType(locationTypes[lt.ToString()], (int)lt));
            }
            comboLocTypeAdd.DataSource = lTypes;
            comboLocTypeAdd.DisplayMember = "DisplayName";
            comboLocTypeAdd.ValueMember = "Value";
            comboLocTypeAdd.SelectedIndex = -1;
        }

        /**
        * Timer event handler
        * Continously polls for WiFi reception, and updates the frontend status.
        * 
        */
        private void timer_Tick(object sender, EventArgs e)
        {
            Main.UpdateWifiIndicator(radioWifiStatus);
        }

        private void menuItemReturn_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void menuItemSave_Click(object sender, EventArgs e)
        {
            if (wifiHelper.signalVector != null && wifiHelper.IsConnected)
            {
                Cursor.Current = Cursors.WaitCursor;
                Location location = new Location();
                location.Name = txtLocNameAdd.Text;
                location.LocationType = (LocationTypes)comboLocTypeAdd.SelectedValue;
                location.Description = txtLocDescAdd.Text;

                Signal[] signals = wifiHelper.signalVector;
                string result = "";
                MessageBoxIcon icon = MessageBoxIcon.None;
                try
                {
                    bool success = locationService.AddLocation(location, signals);
                    result = (success) ? "The location was successfully added!" : "An error occured! The location could not be added.";
                    icon = (success) ? MessageBoxIcon.Asterisk : MessageBoxIcon.Exclamation;
                }
                catch(Exception ex)
                {
                    Cursor.Current = Cursors.Default;
                    MessageBox.Show(ex.Message);
                    return;
                    //Main.ShowConnectionError();
                }
                finally
                {
                    Cursor.Current = Cursors.Default;
                }

                MessageBox.Show(result, "Result", MessageBoxButtons.OK, icon, MessageBoxDefaultButton.Button1);
                txtLocNameAdd.Text = txtLocDescAdd.Text = "";
                comboLocTypeAdd.SelectedIndex = -1;
            }
            else
            {
                Main.ShowConnectionError();
            }
        }

    }
}