﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using MSIS.webservices.user;
using System.Resources;
using System.Globalization;

namespace MSIS
{
    public partial class Login : Form
    {
        private UserProfile user = null;
        private bool userIsAuthenticated = false;

        public Login()
        {
            InitializeComponent();
            Text = AppSettings.AppTitle;

            if (!AppSettings.ConfigFileExists)
            {
                AppSettings.Create();
                ShowConfiguration();
            }

            SetLanguage();

            // Uncomment the following section if the program should require WiFi connection to operate!
            //WiFi wifi = new WiFi();
            //
            //if (wifi.Interface == null)
            //{
            //    MessageBox.Show(Main.ResourceManager.GetString("loginMsgNoWirelessText", AppSettings.Language),
            //                    Main.ResourceManager.GetString("loginMsgNoWirelessTitle", AppSettings.Language),
            //                    MessageBoxButtons.OK, MessageBoxIcon.Exclamation, MessageBoxDefaultButton.Button1);
            //    Application.Exit();
            //}
        }

        private void Login_Load(object sender, EventArgs e)
        {
            if (AppSettings.RememberUsernameAndPassword)
            {
                inputPanel.Enabled = false;
                btnLogin.Focus();
            }
        }

        private void Login_Activated(object sender, EventArgs e)
        {
            // Prefill form data if the user has chosen to remember credentials,
            // or set the textfields to empty values (when a user logs in and afterwards logs out the fields are empty if the program is not supposed to remember user/pass)
            if (AppSettings.RememberUsernameAndPassword)
            {
                if (AppSettings.Username != null) txtUsername.Text = AppSettings.Username;
                if (AppSettings.Password != null) txtPassword.Text = AppSettings.Password;
                this.checkRememberCredentials.Checked = true;
                this.btnLogin.Focus();
                this.inputPanel.Enabled = false;
            }
            else
            {
                txtUsername.Text = "";
                txtPassword.Text = "";
                this.checkRememberCredentials.Checked = false;
            }
        }

        
        #region L10n content
        private void SetLanguage()
        {
            ResourceManager resman = Main.ResourceManager;

            labelTitle.Text = resman.GetString("loginLblTitle", AppSettings.Language);
            labelDesc.Text = resman.GetString("loginLblDescription", AppSettings.Language);
            labelUsername.Text = resman.GetString("loginLblUsername", AppSettings.Language);
            labelPassword.Text = resman.GetString("loginLblPassword", AppSettings.Language);
            checkRememberCredentials.Text = resman.GetString("loginChkRemCredentials", AppSettings.Language);
            btnLogin.Text = resman.GetString("loginBtnLogin", AppSettings.Language);
            menuItemExit.Text = resman.GetString("loginMnuExit", AppSettings.Language);
            menuItemShowSettings.Text = resman.GetString("loginMnuSettings", AppSettings.Language);


        }
        #endregion

        #region Login!
        private void btnLogin_Click(object sender, EventArgs e)
        {
            if (txtUsername.Text.Length > 0 && txtPassword.Text.Length > 0)
            {
                string username = txtUsername.Text;
                string pass = txtPassword.Text;
                // Shall the program remember the user credentials? If so, then store the settings in AppSettings
                if (checkRememberCredentials.Checked)
                {
                    AppSettings.RememberUsernameAndPassword = true;
                    AppSettings.Username = this.txtUsername.Text;
                    AppSettings.Password = this.txtPassword.Text;
                }
                else
                {
                    AppSettings.RememberUsernameAndPassword = false;
                    AppSettings.Username = null;
                    AppSettings.Password = null;
                }

                
                // SHA1 hash
                //System.Security.Cryptography.SHA1 sha = new System.Security.Cryptography.SHA1CryptoServiceProvider();
                //Encoding enc = Encoding.Default;
                //string pass_hash = BitConverter.ToString(sha.ComputeHash(enc.GetBytes(txtPassword.Text))).Replace("-", "");

                // Call Login webservice
                UserService myWebService = new UserService();
                myWebService.Url = AppSettings.WebServiceUrl + "/" + AppSettings.UserServiceName; 
                try
                {
                    Cursor.Current = Cursors.WaitCursor;
                    user = myWebService.Login(username, pass);
                    Cursor.Current = Cursors.Default;
                    if (user == null)
                    {
                        MessageBox.Show(Main.ResourceManager.GetString("loginMsgUnableConnectDBText", AppSettings.Language), Main.ResourceManager.GetString("loginMsgUnableConnectDBTitle", AppSettings.Language),
                            MessageBoxButtons.OK, MessageBoxIcon.Exclamation, MessageBoxDefaultButton.Button1);
                    }
                    else if (user.IsAuthenticated)
                    {
                        // User authenticated OK
                        userIsAuthenticated = true;
                        
                        //Saving User's Current Location ....Uncomment the following section if the connection requires WiFi connection to operate
                       /* WiFi w = new WiFi();
                        webservices.location.LocationService s = new MSIS.webservices.location.LocationService();
                        webservices.location.Location objL = s.GetLocationByFingerprint(w.signalVector);//passing a signal from the wifi
                        s.SaveCurrentLocation(user.Id.ToString(),objL.Id);//pass locationid and userid */

                    }
                    else
                    {
                        MessageBox.Show(Main.ResourceManager.GetString("loginMsgWrongCredText", AppSettings.Language), Main.ResourceManager.GetString("loginMsgWrongCredTitle", AppSettings.Language),
                            MessageBoxButtons.OK, MessageBoxIcon.Asterisk, MessageBoxDefaultButton.Button1);
                    }
                }
                catch (Exception ex)
                {
                    MessageBox.Show(Main.ResourceManager.GetString("loginMsgServerUnavailableText", AppSettings.Language), Main.ResourceManager.GetString("loginMsgServerUnavailableTitle", AppSettings.Language),
                        MessageBoxButtons.OK, MessageBoxIcon.Exclamation, MessageBoxDefaultButton.Button1);
                }
                finally
                {
                    Cursor.Current = Cursors.Default;
                    if (userIsAuthenticated)
                    {
                        // show main menu form
                        new Main(user).ShowDialog();
                    }
                }

            }
        }
        #endregion

        #region Enable input panel on focus
        private void txtUsername_GotFocus(object sender, EventArgs e)
        {
            // enable SIP
            inputPanel.Enabled = true;
        }

        private void txtPassword_GotFocus(object sender, EventArgs e)
        {
            // enable SIP
            inputPanel.Enabled = true;
        }
        #endregion

        #region Configuration 
        private void menuItemShowSettings_Click(object sender, EventArgs e)
        {
            ShowConfiguration();
        }

        private void linkShowConfig_Click(object sender, EventArgs e)
        {
            ShowConfiguration();
        }

        private void ShowConfiguration()
        {
            CultureInfo prelang = AppSettings.Language;
            Configuration config = new Configuration();
            config.ShowDialog();

            if (!prelang.NativeName.Equals(AppSettings.Language.NativeName))
            {
                SetLanguage();
            }

        }
        #endregion
        
        private void menuItemExit_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }

        private void txtUsername_TextChanged(object sender, EventArgs e)
        {

        }

        private void btnNewUser_Click(object sender, EventArgs e)
        {
            new NewUser().ShowDialog();
        }

        private void labelTitle_ParentChanged(object sender, EventArgs e)
        {

        }
    }
}