﻿using System;
using System.Drawing;
using System.IO;
using System.Reflection;
using System.Windows.Forms;

namespace MSIS
{
    /// <summary>
    /// Image control that supports alpha blending.
    /// Use with PNG image files that needs to be displayed on a form that hasn't a white background.
    /// </summary>
    public partial class ZoomButtons : UserControl
    {
        AlphaImage _image;

        public ZoomButtons(Point location)
        {
            InitializeComponent();
            Image = AlphaImage.CreateFromAssemblyResource(Assembly.GetExecutingAssembly(),
                "MSIS.resources.zoom-buttons.png");
            this.Location = location;
        }

        protected override void OnParentChanged(EventArgs e)
        {
            base.OnParentChanged(e);
            // Hook on the paint event on the parent container
            if (Parent != null)
            {
                Parent.Paint += Parent_Paint;
            }
        }

        void Parent_Paint(object sender, PaintEventArgs e)
        {
            // Draw the current control on the parent's surface
            _image.Draw(e.Graphics, new Rectangle(Left, Top, _image.Size.Width + Left, _image.Size.Height + Top));
        }



        /// <summary>
        /// Sets the current <see cref="AlphaImage"/> that the control will display.
        /// </summary>
        public AlphaImage Image
        {
            get
            {
                return _image;
            }
            set
            {
                _image = value;
                // Adjust the control's size to the image size
                Size = _image.Size;
                // Hide ourselves (from being "default-painted" on the parent control)
                Visible = false;
                // Re-paint
                Refresh();
            }
        }

        public void Draw(Graphics g)
        {
            Rectangle destRect = new Rectangle
            (
                Location.X,
                Location.Y,
                Image.Size.Width + Location.X,
                Image.Size.Height + Location.Y
            );
            Image.Draw(g, destRect);
        }

    }
}
