﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using MSIS.webservices.announcements;
using System.Resources;
using System.Globalization;
using System.Collections;

namespace MSIS
{
    public partial class AnnouncementsForm : Form
    {
        string username;
        AnnouncementService aService;
        Announcement[] announcements;
        Course[] courses;
        int offset = 0;

        public AnnouncementsForm(string username)
        {
            InitializeComponent();
            
            this.username = username;

            SetLanguage();
         
            try
            {
                aService = new webservices.announcements.AnnouncementService();
                aService.Url = AppSettings.WebServiceUrl + "/" + AppSettings.AnnouncementServiceName;
            }
            catch
            {
                Main.ShowConnectionError();
                Close();
            }
            finally
            {
                Cursor.Current = Cursors.Default;
            }
        }

        #region L10N content
        private void SetLanguage()
        {
            ResourceManager r = Main.ResourceManager;
            CultureInfo ci = AppSettings.Language;

            // Menu items
            menuItemReturn.Text = r.GetString("newsMnuReturn", ci);
            menuItem2.Text = r.GetString("newsMnuOptions", ci);
            menuItemDateReq.Text = r.GetString("newsMnuDateReq", ci);
            menuItemMaxWeek.Text = r.GetString("newsMnuDateWeek", ci);
            menuItemMaxMonth.Text = r.GetString("newsMnuDateMonth", ci);
            menuItemMaxSemester.Text = r.GetString("newsMnuDateSemester", ci);
            menuItemViewAll.Text = r.GetString("newsMnuDateAll", ci);
            menuItemShow.Text = r.GetString("newsMnuCategories", ci);
            menuItemGeneral.Text = r.GetString("newsMnuCatGeneral", ci);

            // Labels
            lblNoAnnouncements.Text = r.GetString("newsLblNoNews", ci);
            labelHeader.Text = r.GetString("newsLblHeader", ci);
        }
        #endregion

        #region Load, retrieve and display announcements

        private void Announcements_Load(object sender, EventArgs e)
        {
            try
            {
                string[] desiredCourses = AppSettings.Courses;

                // retrieve courses with available rss feeds
                courses = aService.GetCoursesFromIcal(username, true);

                // plan what courses to get feeds for or not (based on user settings)
                if (desiredCourses == null && courses != null)
                {
                    List<string> tmpList = new List<string>();

                    foreach (Course course in courses)
                    {
                        tmpList.Add(course.Code);  
                    }

                    tmpList.Add("general");
                    desiredCourses = tmpList.ToArray();
                }
                else if (desiredCourses == null && courses == null)
                {
                    desiredCourses = new string[] { "all" };
                }

                // retrieve announcements from web service
                //announcements = aService.GetAnnouncements(username, desiredCourses);

                // prepare the menus

                if (courses != null)
                {
                    foreach (Course course in courses)
                    {
                        MenuItem m = new MenuItem();
                        m.Text = course.Code;
                        m.Click += new System.EventHandler(this.CourseMenuItemClicked);

                        menuItemShow.MenuItems.Add(m);
                    }

                }

                // fill the appropriate checkboxes based on retrieved values
                SetCheckedCourses(desiredCourses.ToList());

                // fill the correct date checkbox according to stored settings
                SetCheckedDate();


                AddAnnouncements(desiredCourses);
            }
            catch (Exception ex)
            {
                MessageBox.Show("Could not retrieve announcements");
                Close();
            }

        }

        /// <summary>
        /// Get newsitems from webservice in accordance with user specified options (which categories to get and datefiltered)
        /// </summary>
        /// <param name="desiredCourses">A string array containing the courses to get news from and/or general news items</param>
        private void AddAnnouncements(string[] desiredCourses)
        {
            try
            {
                RemoveAnnouncements();
                announcements = aService.GetAnnouncements(username, desiredCourses, AppSettings.AnnouncementsDateFilter);

                // load the announcements
                if (announcements != null)
                {
                    if (panelAnnouncements.Controls.Contains(lblNoAnnouncements))
                        panelAnnouncements.Controls.Remove(lblNoAnnouncements);
    
                    for (int i = 0; i < announcements.Length; i++)
                    {
                        AddItem(announcements[i]);
                    }
                }
                else
                {
                    if (!panelAnnouncements.Controls.Contains(lblNoAnnouncements))
                        panelAnnouncements.Controls.Add(lblNoAnnouncements);
                }

            }
            catch (Exception ex)
            {
                if (!panelAnnouncements.Controls.Contains(lblNoAnnouncements)) 
                    panelAnnouncements.Controls.Add(lblNoAnnouncements);
                MessageBox.Show("Could not retrieve announcements");
                Close();
            }

            if (Cursor.Current == Cursors.WaitCursor)
                Cursor.Current = Cursors.Default;
        }

        /// <summary>
        /// Remove all announcements displayed in the announcements form.
        /// </summary>
        private void RemoveAnnouncements()
        {
            foreach (Control ctr in this.panelAnnouncements.Controls)
            {
                if (ctr is Label)
                {
                    panelAnnouncements.Controls.Remove(ctr);
                }
            }

            offset = 0;
        }

        private void AddItem(Announcement a)
        {
            // i18n of announcement categories
            string[] cat = Main.ResourceManager.GetString("newsLblAnnouncementTypes", AppSettings.Language).Split('|');
            Dictionary<string, string> cats = new Dictionary<string, string>();

            foreach (string item in cat)
            {
                cats.Add(item.Split('=')[0], item.Split('=')[1]);
            }
            // end of i18n

            // add headline
            Label headLabel = new Label();
            headLabel.ForeColor = Color.Black;
            if (a.Category == AnnouncementCategory.Deadline && (a.DateAdded.CompareTo(DateTime.Today) < 0))
            {
                headLabel.ForeColor = Color.DarkGray;
            }
            headLabel.Font = new Font("Tahoma", 8f, FontStyle.Bold);
            headLabel.Location = new Point(4, 4 + offset);
            string dateMonth = (a.DateAdded.Month.ToString().Length == 1) ? "0" + a.DateAdded.Month : a.DateAdded.Month.ToString();
            string dateDay = (a.DateAdded.Day.ToString().Length == 1) ? "0" + a.DateAdded.Day : a.DateAdded.Day.ToString();
            //headLabel.Text = a.DateAdded.Year + "-" + dateMonth + "-" + dateDay + ": ";
            headLabel.Text = a.Course == null ? "" : a.Course + ": ";
            headLabel.Text += a.Title;
            headLabel.Text += " (" + a.DateAdded.ToLongDateString() + ")";
            headLabel.Size = new Size(panelAnnouncements.Width - 30, 15);
            headLabel.Size = new Size(panelAnnouncements.Width - 30, headLabel.Height * CalculateLabelHeight(headLabel));
            panelAnnouncements.Controls.Add(headLabel);

            // add body text
            Label bodyLabel = new Label();
            bodyLabel.ForeColor = Color.Black;
            if (a.Category == AnnouncementCategory.Deadline && (a.DateAdded.CompareTo(DateTime.Today) < 0))
            {
                bodyLabel.ForeColor = Color.Gray;
            }
            bodyLabel.Font = new Font("Tahoma", 8f, FontStyle.Regular);
            bodyLabel.Location = new Point(4, headLabel.Location.Y + headLabel.Size.Height);
            bodyLabel.Text = "[" + cats[a.Category.ToString()] + "] " + a.Text;
            bodyLabel.Size = new Size(panelAnnouncements.Width - 30, 15);
            bodyLabel.Size = new Size(panelAnnouncements.Width - 30, bodyLabel.Height * CalculateLabelHeight(bodyLabel));
            panelAnnouncements.Controls.Add(bodyLabel);

            // update offset
            offset = (bodyLabel.Location.Y + bodyLabel.Size.Height) + 10;

        }

        private int CalculateLabelHeight(Label label)
        {
            float dpiFactor = AutoScaleDimensions.Height / 96.0F;
            Graphics g = this.CreateGraphics();
            SizeF size = g.MeasureString(label.Text, label.Font);
            int scaleY = (int)(size.Width / (label.Width - 10)) + 1;
            return Convert.ToInt32(scaleY * dpiFactor);
        }
        #endregion

        #region Announcement category selection handlers
        /// <summary>
        /// Handle checking/dechecking of category news items
        /// </summary>
        /// <param name="sender">MenuItem that was pressed</param>
        /// <param name="e">EventArgs</param>
        private void CourseMenuItemClicked(object sender, System.EventArgs e)
        {
            Cursor.Current = Cursors.WaitCursor;
            (sender as MenuItem).Checked = !((sender as MenuItem).Checked);
            AppSettings.Courses = GetCheckedCourses().ToArray();
            AddAnnouncements(GetCheckedCourses().ToArray());
        }

        /// <summary>
        /// Set checked categories in submenu according to stored settings
        /// </summary>
        /// <param name="value"></param>
        private void SetCheckedCourses(List<string> value)
        {
            if (value.Contains("general"))
            {
                menuItemGeneral.Checked = true;
            }
            else if (value.Contains("all"))
            {
                foreach (MenuItem item in menuItemShow.MenuItems)
                {
                    item.Checked = true;
                }
                return;
            }
            else
            {
                menuItemGeneral.Checked = false;
            }


            foreach (MenuItem mnuItem in menuItemShow.MenuItems)
            {
                if (mnuItem != menuItemGeneral)
                    mnuItem.Checked = false;

                foreach (string item in value)
                {
                    if (item.Equals(mnuItem.Text)) mnuItem.Checked = true;
                }
            }
        }

        /// <summary>
        /// Get a listing of all types of news checked in the menu.
        /// </summary>
        /// <returns>A list containing all checked items in the menu</returns>
        private List<string> GetCheckedCourses()
        {
            List<string> courses = new List<string>();

            if (menuItemGeneral.Checked) courses.Add("general");

            foreach (MenuItem item in menuItemShow.MenuItems)
            {
                if (item.Checked && item != menuItemGeneral) courses.Add(item.Text);
            }

            return courses;
        }
        #endregion

        #region Misc event handlers
        private void menuItemReturn_Click(object sender, EventArgs e)
        {
            Close();
        }
        #endregion

        #region Announcement date filter selection
        /// <summary>
        /// Check the menuitem with the date filter corresponding to stored settings
        /// </summary>
        private void SetCheckedDate()
        {
            int days = AppSettings.AnnouncementsDateFilter;

            foreach (MenuItem item in menuItemDateReq.MenuItems)
            {
                item.Checked = false;
            }

            if (days == 0)
            {
                menuItemViewAll.Checked = true;
            }
            else if (days == 7)
            {
                menuItemMaxWeek.Checked = true;
            }
            else if (days == 30)
            {
                menuItemMaxMonth.Checked = true;
            }
            else
            {
                menuItemMaxSemester.Checked = true;
            }
        }

        /// <summary>
        /// Handle the event that occurs when selecting a date filtration in menu
        /// </summary>
        /// <param name="sender">The menuitem that was tapped</param>
        /// <param name="e">EventArgs</param>
        private void DateRequirementClick(object sender, EventArgs e)
        {
            foreach (MenuItem item in menuItemDateReq.MenuItems)
            {
                item.Checked = false;
            }

            (sender as MenuItem).Checked = true;

            int days = 0;

            if (sender == menuItemMaxMonth)
            {
                days = 30;
            }
            else if (sender == menuItemMaxSemester)
            {
                days = -1;
            }
            else if (sender == menuItemMaxWeek)
            {
                days = 7;
            }

            AppSettings.AnnouncementsDateFilter = sender == menuItemMaxSemester ? -1 : days;
            AddAnnouncements(GetCheckedCourses().ToArray());
        }
        #endregion

        private void panelAnnouncements_GotFocus(object sender, EventArgs e)
        {

        }


    }
}