﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Globalization;
using System.Resources;

namespace MSIS
{
    public partial class Configuration : Form
    {
       
        public Configuration()
        {
            InitializeComponent();
            SetLanguage();
        }

        #region L10n content
        private void SetLanguage()
        {
            CultureInfo lang = AppSettings.Language;
            ResourceManager r = Main.ResourceManager;

            this.Text = r.GetString("configFrmTitle", lang);
            this.lblTitle.Text = r.GetString("configLblHeader", lang);
            this.lblDescription.Text = r.GetString("configLblDescription", lang);

            this.lblWebservices.Text = r.GetString("configLblWebservices", lang);
            this.lblMapURL.Text = r.GetString("configLblMapUrl", lang);

            this.lblTooltips.Text = r.GetString("configLblTooltips", lang);
            this.radioShowTooltip.Text = r.GetString("configRadioShowTt", lang);
            this.radioDontShowTooltip.Text = r.GetString("configRadioDontShowTt", lang);

            this.lblLanguage.Text = r.GetString("configLblLang", lang);

            this.menuItemExit.Text = r.GetString("configMnuReturn", lang);
            this.menuItemSave.Text = r.GetString("configMnuSave", lang);

        }
        #endregion

        #region Load settings
        private void Configuration_Load(object sender, EventArgs e)
        {
            if (AppSettings.ConfigFileExists)
            {
                txtWebServicesURL.Text = AppSettings.WebServiceUrl;
                txtMapURL.Text = AppSettings.MapUrl;
                radioShowTooltip.Checked = AppSettings.ShowHints;
                radioDontShowTooltip.Checked = !AppSettings.ShowHints;

                foreach (CultureInfo clt in AppSettings.AvailableLanguages())
                {
                    comboLanguage.Items.Add(clt.TextInfo.ToTitleCase(clt.NativeName));
                }

                comboLanguage.SelectedItem = AppSettings.Language.TextInfo.ToTitleCase(AppSettings.Language.NativeName);
            }
        }
        #endregion

        #region Save settings
        private void menuItemSave_Click(object sender, EventArgs e)
        {
            AppSettings.Create();
            AppSettings.WebServiceUrl = txtWebServicesURL.Text;
            AppSettings.MapUrl = txtMapURL.Text;
            AppSettings.ShowHints = radioShowTooltip.Checked;

            if (!AppSettings.CultureFromName(comboLanguage.SelectedItem.ToString()).NativeName.Equals(AppSettings.Language))
            {
                AppSettings.Language = AppSettings.CultureFromName(comboLanguage.SelectedItem.ToString());
                SetLanguage();
            }

            MessageBox.Show(Main.ResourceManager.GetString("configMsgSuccessText", AppSettings.Language), Main.ResourceManager.GetString("configMsgSuccessTitle", AppSettings.Language),
                MessageBoxButtons.OK, MessageBoxIcon.Asterisk, MessageBoxDefaultButton.Button1);

        }
        #endregion

        private void menuItemExit_Click(object sender, EventArgs e)
        {
            Close();
        }

    }
}