﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using MSIS.webservices.location;

namespace MSIS
{
    /// <summary>
    /// The purpose of this Form is to allow the user to get vital location data easily when debugging or testing the application. This Form should be hidden in Release versions
    /// </summary>
    public partial class FingerprintTest : Form
    {
        private WiFi wifiHelper;
        private LocationService locService;

        public FingerprintTest()
        {
            InitializeComponent();
            
            wifiHelper = new WiFi();
            locService = new LocationService();
        }

        /// <summary>
        /// By default, the form will start by showing Fingerprint data
        /// </summary>
        /// <param name="sender">Sender</param>
        /// <param name="e">EventArgs</param>
        private void FingerprintTest_Load(object sender, EventArgs e)
        {
            GetFingerprintData();
        }

        /// <summary>
        /// Handling menuitem: Return
        /// </summary>
        /// <param name="sender">MenuItem object</param>
        /// <param name="e">EventArgs</param>
        private void menuItemBack_Click(object sender, EventArgs e)
        {
            Close();
        }

        /// <summary>
        /// Handling timer ticks. The timer is active when showing Fingerprint data (Refreshes the content) 
        /// </summary>
        /// <param name="sender">Timer object</param>
        /// <param name="e">EventArgs</param>
        private void timer_Tick(object sender, EventArgs e)
        {
            if (menuItemFingerprint.Checked)
                GetFingerprintData();
        }

       
        /// <summary>
        /// Handle the event that occurs when user presses menu item for showing Fingerprint data
        /// </summary>
        /// <param name="sender">MenuItem object</param>
        /// <param name="e">EventArgs</param>
        private void menuItemFingerprint_Click(object sender, EventArgs e)
        {
            if (!menuItemFingerprint.Checked) 
            { 
                menuItemFingerprint.Checked = true; 
                menuItemCurrent.Checked = false; 
                timer.Enabled = true; 
            }

            GetFingerprintData();
        }

        /// <summary>
        /// Handle the event that occurs when user presses menu item for showing Current Location data
        /// </summary>
        /// <param name="sender">MenuItem object</param>
        /// <param name="e">EventArgs</param>
        private void menuItemCurrent_Click(object sender, EventArgs e)
        {
            if (!menuItemCurrent.Checked) 
            { 
                menuItemCurrent.Checked = true;
                menuItemFingerprint.Checked = false;
                timer.Enabled = false; 
            }

            Cursor.Current = Cursors.WaitCursor;
            GetLocationData();
        }

        /// <summary>
        /// Print out current fingerprint data
        /// </summary>
        private void GetFingerprintData()
        {
            Signal[] signals = wifiHelper.signalVector;
            textFP.Text = "";
            if (signals != null)
            {
                foreach (Signal item in signals)
                {
                    textFP.Text += string.Format("Signal Found: {0} (Signal Strength: {1}, MAC: {2})", item.ssid, item.rssi, item.bssid) + Environment.NewLine;
                }
            }
        }

        /// <summary>
        /// Try to get location or give message otherwise
        /// </summary>
        private void GetLocationData()
        {
            Location loc = GetFingerprintLocation();

            if (loc != null)
            {
                textFP.Text = "The service has approximated your location to be:" + Environment.NewLine + Environment.NewLine + loc.Name;
            }
            else
            {
                textFP.Text = "The service was unable to get a location matching your fingerprint. Check that you are connected to a WiFi network, or try to move to another location.";
            }

            Cursor.Current = Cursors.Default;
        }

        /// <summary>
        /// Retrieve Location from Web Service using Fingerprint Signal Vector
        /// </summary>
        /// <returns>Location data</returns>
        private Location GetFingerprintLocation()
        {
            Signal[] signals = null;

            try
            {
                signals = wifiHelper.signalVector;
            }
            catch (Exception)
            {
                return null;
            }

            if (signals != null)
            {
                try
                {
                    Location location = locService.GetLocationByFingerprint(signals);
                    if (location != null)
                    {
                        return location;
                    }
                }
                catch (Exception e)
                {
                    return null;
                }
            }

           return null;
        }

    }
}