﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Reflection;
using OpenNETCF.Net.NetworkInformation;
using MSIS.webservices.user;
using System.Resources;
using System.Globalization;

namespace MSIS
{
    public partial class Main : Form
    {
        // Set this to true to enable menu items: Add location & Fingerprint test
        bool developerVersion = false;

        UserProfile user;
        static WiFi wifiHelper = new WiFi();
        Image tmpImage;

        public Main(UserProfile user)
        {
            InitializeComponent();
            Text = AppSettings.AppTitle;

            radioWifiStatus.Text = ResourceManager.GetString("mainWifiStatusRetrieving", AppSettings.Language);

            this.user = user;
            lblWelcome.Text = ResourceManager.GetString("mainLblWelcomeUser", AppSettings.Language) + " " + user.Name + "!";

            // Remove administrative menu items in standard versions
            if (!developerVersion)
            {
                menuItem2.MenuItems.Remove(menuItemAddLocation);
                menuItem2.MenuItems.Remove(menuItemFP);
            }
            
            SetLanguage();
        }

        #region L10N content
        private void SetLanguage()
        {
            mMenuAnnouncements.Image = new Bitmap(System.Reflection.Assembly.GetExecutingAssembly().GetManifestResourceStream(
                        "MSIS.resources.announcements-" + AppSettings.Language.ToString() + ".png"));
            mMenuSearch.Image = new Bitmap(System.Reflection.Assembly.GetExecutingAssembly().GetManifestResourceStream(
                        "MSIS.resources.location-search-" + AppSettings.Language.ToString() + ".png"));
            mMenuSchedule.Image = new Bitmap(System.Reflection.Assembly.GetExecutingAssembly().GetManifestResourceStream(
                        "MSIS.resources.schedule-" + AppSettings.Language.ToString() + ".png"));

            labelWelcome.Text = ResourceManager.GetString("mainLblWelcome", AppSettings.Language);
            lblMenuDescription.Text = ResourceManager.GetString("mainLblDescriptionGeneral", AppSettings.Language);
            menuItem1.Text = ResourceManager.GetString("mainMnuExit", AppSettings.Language);
            menuItemExit.Text = ResourceManager.GetString("mainMnuExit", AppSettings.Language);
            menuItemLogout.Text = ResourceManager.GetString("mainMnuLogout", AppSettings.Language);
            menuItemSettings.Text = ResourceManager.GetString("mainMnuSettings", AppSettings.Language);
            menuItemAddLocation.Text = ResourceManager.GetString("mainMnuAddLocation", AppSettings.Language);
            menuItem2.Text = ResourceManager.GetString("mainMnu", AppSettings.Language);
        }
        #endregion

        #region Program public methods for updating the WiFi-indicator, getting the ResourceManager, displaying connection error message and listing enum values
        public static void UpdateWifiIndicator(RadioButton indicator)
        {
            if (wifiHelper.IsConnected)
            {
                Dictionary<string, string> signals = new Dictionary<string, string>();
                string[] tmp = Main.ResourceManager.GetString("globalWifiSignal", AppSettings.Language).Split('|');

                foreach (string item in tmp)
                {
                    signals.Add(item.Split('=')[0], item.Split('=')[1]);
                }

                indicator.Checked = true;
                indicator.Enabled = true;

                string signalStrength = "";
                try
                {
                    signalStrength = " (" + signals[wifiHelper.Interface.SignalStrength.ToString()] + ")";
                }
                catch (Exception e) { }

                indicator.Text = ResourceManager.GetString("mainWifiStatusConnected", AppSettings.Language)
                    + " '" + wifiHelper.Interface.AssociatedAccessPoint + "'" + signalStrength;
            }
            else
            {
                indicator.Checked = false;
                indicator.Enabled = false;
                indicator.Text = ResourceManager.GetString("mainWifiNoConnection", AppSettings.Language);
            }
        }

        private static ResourceManager resman;

        public static ResourceManager ResourceManager
        {
            get
            {
                if (resman == null)
                {
                    resman = new ResourceManager("MSIS.resource", System.Reflection.Assembly.GetExecutingAssembly());
                }

                return resman;
            }
        }

        public static void ShowConnectionError()
        {
            MessageBox.Show(ResourceManager.GetString("mainMsgConnectionErrorText", AppSettings.Language),
                ResourceManager.GetString("mainMsgConnectionErrorTitle", AppSettings.Language), MessageBoxButtons.OK, MessageBoxIcon.Exclamation, MessageBoxDefaultButton.Button1);
        }

        public static IEnumerable<Enum> GetEnumValues(Enum enumeration)
        {
            List<Enum> enumerations = new List<Enum>();
            foreach (FieldInfo fieldInfo in enumeration.GetType().GetFields(BindingFlags.Static | BindingFlags.Public))
            {
                enumerations.Add((Enum)fieldInfo.GetValue(enumeration));
            }
            return enumerations;
        }
        #endregion

        #region Menu/MenuItem event handling
        private void menuItemExit_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }

        private void menuItemLogout_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void btnAnnouncements_Click(object sender, EventArgs e)
        {
            Cursor.Current = Cursors.WaitCursor;
            new AnnouncementsForm(user.Username).ShowDialog();
            
        }

        private void btnSchedule_Click(object sender, EventArgs e)
        {
            Cursor.Current = Cursors.WaitCursor;
            new Schedule(user.Username).ShowDialog();
        }

        private void btnLocationFinder_Click(object sender, EventArgs e)
        {
            Cursor.Current = Cursors.WaitCursor;
            new LocationFinder().ShowDialog();
        }

        private void menuItemSettings_Click(object sender, EventArgs e)
        {
            CultureInfo prelang = AppSettings.Language;
            Configuration config = new Configuration();
            config.ShowDialog();

            if (!prelang.NativeName.Equals(AppSettings.Language.NativeName))
            {
                SetLanguage();
            }
        }

        private void menuItemAddLocation_Click(object sender, EventArgs e)
        {
            if (wifiHelper.IsConnected)
            {
                LocationManager lm = new LocationManager();
                lm.ShowDialog();
            }
            else
            {
                ShowConnectionError();
            }
        }

        private void menuItemFP_Click(object sender, EventArgs e)
        {
            FingerprintTest fp = new FingerprintTest();
            fp.ShowDialog();
        }

        private void mMenuItem_Hover(object sender, MouseEventArgs e)
        {
            switch ((sender as PictureBox).Name)
            {
                case "mMenuAnnouncements":
                    lblMenuDescription.Text = ResourceManager.GetString("mainLblDescriptionAnnouncements", AppSettings.Language);
                    tmpImage = mMenuAnnouncements.Image;

                    mMenuAnnouncements.Image = new Bitmap(System.Reflection.Assembly.GetExecutingAssembly().GetManifestResourceStream(
                        "MSIS.resources.announcements-hover-" + AppSettings.Language.ToString() + ".png"));
                    break;
                case "mMenuSearch":
                    lblMenuDescription.Text = ResourceManager.GetString("mainLblDescriptionMySchedule", AppSettings.Language);
                    tmpImage = mMenuSearch.Image;
                    mMenuSearch.Image = new Bitmap(System.Reflection.Assembly.GetExecutingAssembly().GetManifestResourceStream(
                        "MSIS.resources.location-search-hover-" + AppSettings.Language.ToString() + ".png"));
                    break;
                case "mMenuSchedule":
                    lblMenuDescription.Text = ResourceManager.GetString("mainLblDescriptionLocationSearch", AppSettings.Language);
                    tmpImage = mMenuSchedule.Image;
                    mMenuSchedule.Image = new Bitmap(System.Reflection.Assembly.GetExecutingAssembly().GetManifestResourceStream(
                        "MSIS.resources.schedule-hover-" + AppSettings.Language.ToString() + ".png"));
                    break;
            }
        }

        private void mMenuItem_MouseUp(object sender, MouseEventArgs e)
        {
            lblMenuDescription.Text = ResourceManager.GetString("mainLblDescriptionGeneral", AppSettings.Language);
            (sender as PictureBox).Image = tmpImage;
        }
        #endregion

        #region Timer event handling for displaying WiFi connectivity
        private void timer_Tick(object sender, EventArgs e)
        {
            UpdateWifiIndicator(radioWifiStatus);
        }
        #endregion

        private void menuItem2_Click(object sender, EventArgs e)
        {

        }

        

       

        private void pictureBox1_Click_2(object sender, EventArgs e)
        {
            Cursor.Current = Cursors.WaitCursor;
            new ResourceForm(user.Id).ShowDialog();
        }

        private void mMenuCollaboration_Click(object sender, EventArgs e)
        {
            //Cursor.Current = Cursors.WaitCursor;
           
            new GroupForm(user).ShowDialog();
        }

        

        

    }
}