﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.Windows.Forms;
using System.IO;

namespace MSIS
{
    static class Program
    {
        /// <summary>
        /// The main entry point for the application.
        /// </summary>
        [MTAThread]
        static void Main()
        {
            RenameResourceFiles();
            Application.Run(new Login());
        }

        /// <summary>
        /// In order to work around a bug in CabWiz installation utility we install the language files as MSIS.resources.langname.dll instead of
        /// MSIS.resources.dll. These files need to be renamed before we can use them, and this method is intended for exactly that. When starting
        /// up the program we check if any files exists with the langname in it, which could mean that the program has been upgraded, reinstalled
        /// or something else. Also in upgrade scenarios it may happen that both files exist. We then need to remove the 'old' resource files before we rename them.
        /// </summary>
        static void RenameResourceFiles()
        {
            string AppPath = Path.GetDirectoryName(System.Reflection.Assembly.GetExecutingAssembly().GetName().CodeBase);
            string en = AppPath + "\\en\\MSIS.resources.en.dll";
            string nbno = AppPath + "\\nb-NO\\MSIS.resources.nbno.dll";

            if (File.Exists(en))
            {
                // delete any pre-existing files
                if (File.Exists(AppPath + "\\en\\MSIS.resources.dll"))
                {
                    File.Delete(AppPath + "\\en\\MSIS.resources.dll");
                }

                File.Move(en, AppPath+"\\en\\MSIS.resources.dll");
            }

            if (File.Exists(nbno))
            {
                // delete any pre-existing files
                if (File.Exists(AppPath + "\\nb-NO\\MSIS.resources.dll"))
                {
                    File.Delete(AppPath + "\\nb-NO\\MSIS.resources.dll");
                }

                File.Move(nbno, AppPath + "\\nb-NO\\MSIS.resources.dll");
            }
        }
    }
}