﻿using System;
using System.Globalization;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using MSIS.webservices.schedule;
using System.Resources;
using System.Collections;

namespace MSIS
{
    
    public partial class Schedule : Form
    {
       
        CultureInfo cCulture = CultureInfo.CurrentCulture;
        DateTime currentDate;
        int weekNumber;
        int selTabIdx;
        string username; // username of logged in user
        enum CalendarView { Lectures, Exercises, Overview };
        CalendarView currentView = CalendarView.Lectures;
        Panel[] viewPanels;
        MenuItem[] viewMenuItems;
        int tmpClickedAppointmentIdx = -1;

        // Remote fields/objects
        ScheduleService scheduleService;
        Appointment[] appointments; // all calendar appointments for user

        public Schedule(string username)
        {
            InitializeComponent();
            Text = AppSettings.AppTitle;

            lblHeader.Text = Main.ResourceManager.GetString("scheduleHeader", AppSettings.Language);
           
            this.username = username;
            try
            {
                scheduleService = new webservices.schedule.ScheduleService();
                scheduleService.Url = AppSettings.WebServiceUrl + "/" + AppSettings.ScheduleServiceName;
            }
            catch
            {
                Main.ShowConnectionError();
                Close();
            }
            finally
            {
                Cursor.Current = Cursors.Default;
            }

            currentDate = DateTime.Now; 
            SetLanguage();

            viewPanels = new Panel[] { panelLectures, panelWeekOverview };
            viewMenuItems = new MenuItem[] { menuShowLectures, menuShowExercises, menuShowWeekOverview };

            panelLectures.Width = panelBottomContainer.Width-panelBorderVr.Width-panelHours.Width;
            panelWeekOverview.Width = panelLectures.Width;
            
            if (AppSettings.ShowHints)
            {
                MessageBox.Show(Main.ResourceManager.GetString("scheduleMsgTipText", AppSettings.Language), Main.ResourceManager.GetString("scheduleMsgTipTitle", AppSettings.Language), MessageBoxButtons.OK,
                    MessageBoxIcon.Asterisk, MessageBoxDefaultButton.Button1);
            }
        }

        private void Schedule_Load(object sender, EventArgs e)
        {
            try
            {
                appointments = scheduleService.GetLecturesFromIcal(username, currentDate);
            }
            catch (Exception es)
            {
                Main.ShowConnectionError();
            }

            if (currentView == CalendarView.Lectures)
            {
                ToggleDateLabels(false);
                showView(panelLectures);
                lblHeader.Text = Main.ResourceManager.GetString("scheduleHeaderLectures", AppSettings.Language);
            }
            else if (currentView == CalendarView.Overview)
            {
                CreateWeeklySummary();
                showView(panelWeekOverview);
                lblHeader.Text = Main.ResourceManager.GetString("scheduleHeaderWeekly", AppSettings.Language);
            }
        }


        #region L10N content
        private void SetLanguage()
        {
            ResourceManager res = Main.ResourceManager;
            CultureInfo clt = AppSettings.Language;

            // Link labels
            SetWeekNumbers(res, clt);

            // Day labels
            lblMon.Text = res.GetString("scheduleDayMon", clt);
            lblTue.Text = res.GetString("scheduleDayTue", clt);
            lblWed.Text = res.GetString("scheduleDayWed", clt);
            lblThu.Text = res.GetString("scheduleDayThu", clt);
            lblFri.Text = res.GetString("scheduleDayFri", clt);

            // Day tabs
            tabControl.TabPages[0].Text = res.GetString("scheduleDayMon", clt);
            tabControl.TabPages[1].Text = res.GetString("scheduleDayTue", clt);
            tabControl.TabPages[2].Text = res.GetString("scheduleDayWed", clt);
            tabControl.TabPages[3].Text = res.GetString("scheduleDayThu", clt);
            tabControl.TabPages[4].Text = res.GetString("scheduleDayFri", clt);
            tabControl.TabPages[5].Text = res.GetString("scheduleDaySat", clt);
            tabControl.TabPages[6].Text = res.GetString("scheduleDaySun", clt);

            // Menu items
            menuItemReturn.Text = res.GetString("scheduleMnuReturn", clt);
            menuChangeView.Text = res.GetString("scheduleMnuChangeView", clt);
            menuShowExercises.Text = res.GetString("scheduleMnuExercises", clt);
            menuShowLectures.Text = res.GetString("scheduleMnuLectures", clt);
            menuShowWeekOverview.Text = res.GetString("scheduleMnuWeekly", clt);
            menuItemShowExams.Text = res.GetString("scheduleMnuExams", clt);

            // Context menu
            menuItemShowDetails.Text = res.GetString("scheduleContextDetails", clt);
            menuItemShowMap.Text = res.GetString("scheduleContextMap", clt);

        }

        private void SetWeekNumbers(ResourceManager res, CultureInfo clt)
        {
            linkLabelNextWeek.Text = ">>" + Environment.NewLine + res.GetString("scheduleWeek", clt) + Environment.NewLine + clt.Calendar.GetWeekOfYear(currentDate.AddDays(7), CalendarWeekRule.FirstFourDayWeek, System.DayOfWeek.Monday);
            linkLabelPrevWeek.Text = "<<" + Environment.NewLine + res.GetString("scheduleWeek", clt) + Environment.NewLine + clt.Calendar.GetWeekOfYear(currentDate.Subtract(new TimeSpan(7, 0, 0, 0)), CalendarWeekRule.FirstFourDayWeek, System.DayOfWeek.Monday);
        }
        #endregion

        #region Appointment handling
        private void tabControl_SelectedIndexChanged(object sender, EventArgs e)
        {
            // run only if target tab is different from current tab
            if (selTabIdx != (sender as TabControl).SelectedIndex)
            {
                currentDate = currentDate.AddDays((sender as TabControl).SelectedIndex - selTabIdx);
                SetDateTab(currentDate);
            }
        }

        private void ToggleDateLabels(bool enable)
        {
            //lblDate.Visible = !enable;
            lblMon.Visible = enable;
            lblTue.Visible = enable;
            lblWed.Visible = enable;
            lblThu.Visible = enable;
            lblFri.Visible = enable;
            tabControl.Enabled = !enable;

            if (!enable)
            {
                SetDateTab(currentDate);
            }
        }

        private void SetDateTab(DateTime date)
        {
            // Workaround for 'week 53' bug
            DateTime dateFixed = (date.DayOfWeek >= System.DayOfWeek.Monday && date.DayOfWeek <= System.DayOfWeek.Wednesday) ? date.AddDays(3) : date;
            weekNumber = cCulture.Calendar.GetWeekOfYear(dateFixed, CalendarWeekRule.FirstFourDayWeek, System.DayOfWeek.Monday);
            // Update date label
            lblDate.Text = "(" + Main.ResourceManager.GetString("scheduleWeek", AppSettings.Language) + " " + weekNumber + " - " + date.ToLongDateString() + ")"; //date.ToString("MMMM", CultureInfo.CreateSpecificCulture("en-US")) + " " + date.Day + ", " + date.Year + ")";

            // Select correct day tab
            int dayOfWeek = (int)date.DayOfWeek - 1;
            selTabIdx = (dayOfWeek < 0) ? 6 : dayOfWeek;
            tabControl.SelectedIndex = selTabIdx;
            if (date == DateTime.Today && !tabControl.TabPages[selTabIdx].Text.Contains(Main.ResourceManager.GetString("scheduleToday", AppSettings.Language)))
            {
                tabControl.TabPages[selTabIdx].Text += " (" + Main.ResourceManager.GetString("scheduleToday", AppSettings.Language) + ")";
            }

            try
            {
                appointments = scheduleService.GetLecturesFromIcal(username, date);
            }
            catch
            {
                Main.ShowConnectionError();
            }
            finally
            {
                // Load appointments for current date/day tab,
                // first clear all appointment labels from panels
                panelLectures.Controls.Clear();

                if (appointments != null)
                {
                    
                    for (int i = 0; i < appointments.Length; i++)
                    {
                        Appointment a = appointments[i];

                        // Add only appointments where date and week number match
                        //bool thisWeek = Array.IndexOf<int>(a.Weeks, weekNumber) != -1 ? true : false;
                        if ((int)a.Day == (int)date.DayOfWeek) // && thisWeek)
                        {
                            AddAppointment(a, i);
                        }
                    }
                }
            }
        }

        /// <summary>
        /// Add an appointment to the lecture or exercise panels.
        /// </summary>
        /// <param name="appointment">The appointment to add</param>
        /// <param name="idx">The index of the appointment</param>
        private void AddAppointment(Appointment appointment, int idx)
        {
            float scalingFactorX = AutoScaleDimensions.Width / 96.0F;
            float scalingFactorY = AutoScaleDimensions.Height / 96.0F;
            LinkLabel aptLabel = new LinkLabel();
            aptLabel.BackColor = Color.FromArgb(77, 108, 131);
            aptLabel.ForeColor = Color.White;
            aptLabel.Font = new Font("Tahoma", 7.0f, FontStyle.Bold);
            aptLabel.Anchor = AnchorStyles.Top | AnchorStyles.Left;
            aptLabel.Name = "idx_" + idx;
            aptLabel.Click += new EventHandler(aptLabel_Click);

            // calculate height of label based on appointment duration
            int lblHeight = Convert.ToInt32((25 / 60.0) * appointment.Duration) + 7;

            // calculate position of label based on starting time of appointment
            int lblYCoord = Convert.ToInt32((appointment.StartHour - 8) * 25 + appointment.StartMinute * (25 / 60.0));

            aptLabel.Bounds = new Rectangle(3, lblYCoord, panelLectures.Width - 6, lblHeight);
            // scale to current DPI
            aptLabel.Scale(new SizeF(1f, scalingFactorY));

            if (lblHeight >= 40)
            {
                aptLabel.Text = appointment.Title + Environment.NewLine;

                aptLabel.Text += Main.ResourceManager.GetString("scheduleDetailsRoom", AppSettings.Language) + ": " + appointment.Room;

                string startHr = (appointment.StartHour.ToString().Length == 1) ? "0" + appointment.StartHour : appointment.StartHour.ToString();
                string startMn = (appointment.StartMinute.ToString().Length == 1) ? "0" + appointment.StartMinute : appointment.StartMinute.ToString();
                string endHr = (appointment.EndHour.ToString().Length == 1) ? "0" + appointment.EndHour : appointment.EndHour.ToString();
                string endMn = (appointment.EndMinute.ToString().Length == 1) ? "0" + appointment.EndMinute : appointment.EndMinute.ToString();

                aptLabel.Text += Environment.NewLine + startHr + ":" + startMn + " - " + endHr + ":" + endMn;
            }
            else
            {
                aptLabel.Text = (appointment.Title.Length > 35 ? appointment.Title.Substring(0, 35) + "..." : appointment.Title) + Environment.NewLine + Main.ResourceManager.GetString("scheduleDetailsRoom", AppSettings.Language) + ": " + appointment.Room;
            }

            // Experimental... Workaround for overlapping appointments
            foreach (Control control in panelLectures.Controls)
            {
                if (control is Label)
                {
                    if ((control as Label).Location.Y == aptLabel.Location.Y)
                    {
                        aptLabel.Location = new Point(aptLabel.Location.X, aptLabel.Location.Y + 5);
                    }
                }
            }
            
         
            if (appointment.AppointmentType == MSIS.webservices.schedule.Type.Lecture && menuShowLectures.Checked)
            {
                panelLectures.Controls.Add(aptLabel);
            }
            else if (appointment.AppointmentType == MSIS.webservices.schedule.Type.Exercise && menuShowExercises.Checked)
            {
                panelLectures.Controls.Add(aptLabel);
            }
            else if (appointment.AppointmentType == MSIS.webservices.schedule.Type.Exam && menuItemShowExams.Checked)
            {
                panelLectures.Controls.Add(aptLabel);
            }

            

            //if (appointment.AppointmentType == MSIS.webservices.schedule.Type.Lecture || appointment.AppointmentType == MSIS.webservices.schedule.Type.Other)
            //{
            //    panelLectures.Controls.Add(aptLabel);
            //}
            //if (appointment.AppointmentType == MSIS.webservices.schedule.Type.Exercise)
            //{
            //    panelExercises.Controls.Add(aptLabel);
            //}

            aptLabel.BringToFront();

        }
        #endregion

        #region Weekly summary
        private void CreateWeeklySummary()
        {
            panelWeekOverview.Controls.Clear();
            weekNumber = cCulture.Calendar.GetWeekOfYear(currentDate, CalendarWeekRule.FirstFourDayWeek, System.DayOfWeek.Monday);
            lblDate.Text = "(" + Main.ResourceManager.GetString("scheduleWeek", AppSettings.Language) + " " + weekNumber + ")";

            if (appointments == null || (cCulture.Calendar.GetWeekOfYear(currentDate, CalendarWeekRule.FirstFourDayWeek, System.DayOfWeek.Monday) != cCulture.Calendar.GetWeekOfYear(appointments[0].Date, CalendarWeekRule.FirstFourDayWeek, System.DayOfWeek.Monday)))
            {
                try { appointments = scheduleService.GetLecturesFromIcal(username, currentDate); }
                catch { return; }
            }

            if (appointments == null) return;

            ToggleDateLabels(true);

            float scalingFactorX = AutoScaleDimensions.Width / 96.0F;
            float scalingFactorY = AutoScaleDimensions.Height / 96.0F;


            for (int i = 0; i < appointments.Length; i++)
            {
                Appointment appointment = appointments[i];


                // Label for week summary
                LinkLabel aptLabelOverview = new LinkLabel();
                aptLabelOverview.BackColor = Color.FromArgb(77, 108, 131);
                aptLabelOverview.ForeColor = Color.White;
                aptLabelOverview.Font = new Font("Tahoma", 7.0f, FontStyle.Bold);
                aptLabelOverview.Anchor = AnchorStyles.Top | AnchorStyles.Left;
                aptLabelOverview.Name = "idx_" + i; // index of appointment
                aptLabelOverview.Click += new EventHandler(aptLabel_Click);
                aptLabelOverview.TextAlign = ContentAlignment.TopCenter;

                // calculate height of label based on appointment duration
                int lblHeight = Convert.ToInt32((25 / 60.0) * appointment.Duration) + 7;

                // calculate position of label based on starting time of appointment
                int lblYCoord = Convert.ToInt32((appointment.StartHour - 8) * 25 + appointment.StartMinute * (25 / 60.0));

                int xPos = ((int)appointment.Day - 1) * 8 + ((int)appointment.Day - 1) * 30 + 6;
                aptLabelOverview.Bounds = new Rectangle(xPos, lblYCoord, 28, lblHeight); // was 28
                aptLabelOverview.Scale(new SizeF(scalingFactorX, scalingFactorY));
                if (appointment.AppointmentType == MSIS.webservices.schedule.Type.Lecture)
                {
                    aptLabelOverview.Text = string.Format("*{0}*", Main.ResourceManager.GetString("scheduleSummaryLecture", AppSettings.Language));
                }
                else if (appointment.AppointmentType == MSIS.webservices.schedule.Type.Exercise)
                {
                    aptLabelOverview.Text = string.Format("*{0}*", Main.ResourceManager.GetString("scheduleSummaryExercise", AppSettings.Language));
                }
                else if (appointment.AppointmentType == MSIS.webservices.schedule.Type.Exam)
                {
                    aptLabelOverview.Text = string.Format("*{0}*", Main.ResourceManager.GetString("scheduleSummaryExam", AppSettings.Language));
                }
                else if (appointment.AppointmentType == MSIS.webservices.schedule.Type.Other)
                {
                    aptLabelOverview.Text = "*" + Main.ResourceManager.GetString("scheduleSummaryLecture", AppSettings.Language) + "/" + Main.ResourceManager.GetString("scheduleSummaryExercise", AppSettings.Language) + "*";
                }
                if (appointment.Room.Length < 6)
                {
                    aptLabelOverview.Text += Environment.NewLine + appointment.Room;
                }

                // Add label to week overview
                panelWeekOverview.Controls.Add(aptLabelOverview);
                aptLabelOverview.BringToFront();

            }
        }
        #endregion

        #region Context menu and viewtype event handling
        void aptLabel_Click(object sender, EventArgs e)
        {
            LinkLabel lbl = sender as LinkLabel;
            tmpClickedAppointmentIdx = Int32.Parse(lbl.Name.Split('_')[1]);
            contextMenu.Show(this, new Point(lbl.Bounds.Right, lbl.Bounds.Bottom));
        }

        private void contextMenu_Click(object sender, EventArgs e)
        {
            Object item = sender;

            if (item == menuItemShowDetails && tmpClickedAppointmentIdx >= 0)
            {
                Appointment appointment = appointments[tmpClickedAppointmentIdx];
                
                string startHr = (appointment.StartHour.ToString().Length == 1) ? "0" + appointment.StartHour : appointment.StartHour.ToString();
                string startMn = (appointment.StartMinute.ToString().Length == 1) ? "0" + appointment.StartMinute : appointment.StartMinute.ToString();
                string endHr = (appointment.EndHour.ToString().Length == 1) ? "0" + appointment.EndHour : appointment.EndHour.ToString();
                string endMn = (appointment.EndMinute.ToString().Length == 1) ? "0" + appointment.EndMinute : appointment.EndMinute.ToString();
                string hours = startHr + ":" + startMn + " - " + endHr + ":" + endMn;

                string teacher = (appointment.Person != "") ? appointment.Person : Main.ResourceManager.GetString("scheduleLecturerUnknown", AppSettings.Language);

                string type = "";
                if (appointment.AppointmentType == MSIS.webservices.schedule.Type.Other)
                {
                    type = Main.ResourceManager.GetString("scheduleDetailsLecture", AppSettings.Language) + "/" + Main.ResourceManager.GetString("scheduleDetailsExercise", AppSettings.Language);
                }
                else if (appointment.AppointmentType == MSIS.webservices.schedule.Type.Lecture)
                {
                    type = Main.ResourceManager.GetString("scheduleDetailsLecture", AppSettings.Language);
                }
                else if (appointment.AppointmentType == MSIS.webservices.schedule.Type.Exam)
                {
                    type = Main.ResourceManager.GetString("scheduleDetailsExam", AppSettings.Language);
                }
                else
                {
                    type = Main.ResourceManager.GetString("scheduleDetailsExercise", AppSettings.Language);
                }

                string[] weekdays = Main.ResourceManager.GetString("scheduleWeekDays", AppSettings.Language).Split(',');
                //string type = (appointment.AppointmentType == MSIS.webservices.schedule.Type.Other) ? Main.ResourceManager.GetString("scheduleDetailsLecture", AppSettings.Language) + "/" + Main.ResourceManager.GetString("scheduleDetailsExercise", AppSettings.Language) : appointment.AppointmentType.ToString();

                string detailsText =
                Main.ResourceManager.GetString("scheduleDetailsCourse", AppSettings.Language) + ": " + appointment.Title + Environment.NewLine +
                Main.ResourceManager.GetString("scheduleDetailsType", AppSettings.Language) + ": " + type + Environment.NewLine +
                Main.ResourceManager.GetString("scheduleDetailsTime", AppSettings.Language) + ": " + weekdays[Convert.ToInt32(appointment.Day) - 1] + " " + hours + Environment.NewLine +
                Main.ResourceManager.GetString("scheduleDetailsRoom", AppSettings.Language) + ": " + appointment.Room + Environment.NewLine;
                
                PopUp detailsPopUp = new PopUp(detailsText);
                detailsPopUp.Title = Main.ResourceManager.GetString("scheduleDetailsTitle", AppSettings.Language);
                detailsPopUp.ShowDialog();
            }
            else if (item == menuItemShowMap)
            {
                string locRoom = appointments[tmpClickedAppointmentIdx].Room;
                MapWindow mapWindow = new MapWindow(locRoom, true);
            }
        }

        private void menuToggleScheduleView_Click(object sender, EventArgs e)
        {
            MenuItem menuItem = (sender as MenuItem);

            if (menuItem == menuShowExercises || menuItem == menuShowLectures || menuItem == menuItemShowExams)
            {
                ToggleDateLabels(false);

                menuItem.Checked = !menuItem.Checked;

                currentView = CalendarView.Lectures;
                showView(panelLectures);
                lblHeader.Text = Main.ResourceManager.GetString("scheduleHeader", AppSettings.Language);
            }
            else if (menuItem == menuShowWeekOverview)
            {
                currentView = CalendarView.Overview;
                CreateWeeklySummary();
                showView(panelWeekOverview);
                lblHeader.Text = Main.ResourceManager.GetString("scheduleHeaderWeekly", AppSettings.Language);
                // Update date label
                weekNumber = cCulture.Calendar.GetWeekOfYear(currentDate, CalendarWeekRule.FirstFourDayWeek, System.DayOfWeek.Monday);
                lblDate.Text = "(" + Main.ResourceManager.GetString("scheduleWeek", AppSettings.Language) + " " + weekNumber + ")";
            }
        }

        private void showView(Panel activePanel)
        {
            if (activePanel == panelLectures)
            {
                panelLectures.Visible = true;
                SetDateTab(currentDate);
            }
            else
            {
                panelLectures.Visible = false;
                menuShowLectures.Checked = false;
                menuShowExercises.Checked = false;
                menuItemShowExams.Checked = false;
            }

            if (activePanel == panelWeekOverview)
            {
                panelWeekOverview.Visible = true;
                menuShowWeekOverview.Checked = true;
            }
            else
            {
                panelWeekOverview.Visible = false;
                menuShowWeekOverview.Checked = false;
            }
        }
        #endregion

        #region Misc event handlers
        private void menuItemReturn_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void linkLabelPrevWeek_Click(object sender, EventArgs e)
        {
            currentDate = currentDate.Subtract(new TimeSpan(7, 0, 0, 0));
            if (currentView == CalendarView.Exercises || currentView == CalendarView.Lectures)
            {
                SetDateTab(currentDate);
            }
            else
            {
                CreateWeeklySummary();
            }

            SetWeekNumbers(Main.ResourceManager, AppSettings.Language);
        }

        private void linkLabelNextWeek_Click(object sender, EventArgs e)
        {
            currentDate = currentDate.AddDays(7);
            if (currentView == CalendarView.Exercises || currentView == CalendarView.Lectures)
            {
                SetDateTab(currentDate);
            }
            else
            {
                CreateWeeklySummary();
            }

            SetWeekNumbers(Main.ResourceManager, AppSettings.Language);
        }

        #endregion

    }
}