﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.Text;
using OpenNETCF.Net.NetworkInformation;
using MSIS.webservices.location;
namespace MSIS
{
    class WiFi
    {
        WirelessZeroConfigNetworkInterface nic = null;
        int numberOfAPs = 3; // number of Access Points to use in calculations

        public WiFi()
        {
            
            NetworkInterface[] nics = (NetworkInterface[]) NetworkInterface.GetAllNetworkInterfaces(); // get all NICS
            foreach (NetworkInterface iface in nics)
            {
                if (iface is WirelessZeroConfigNetworkInterface)
                {
                    nic = (WirelessZeroConfigNetworkInterface)iface;
                }
            }
        }

        /// <summary>
        /// Returns a vector of bssid, ssid and rssi of the numberOfAPs APs with best signal strength.
        /// Default is 3 APs. Might need minor adjustment to other classes to modify.
        /// Strongest signal is first element
        /// </summary>
        public Signal[] signalVector
        {
            get // NOTE: might have to explicitly define signal vector 3 elements long
            {
                if (nic != null)
                {
                    AccessPointCollection availableAPs = nic.NearbyAccessPoints;
                    if (availableAPs == null || availableAPs.Count <= 0)
                    {
                        return null;
                    }
                    // strongest signal (lowest -db value) at the front of list
                    List<Signal> signalVector = new List<Signal>();
                    for (int i = 0; i < availableAPs.Count; i++)
                    {
                        if (availableAPs[i].InfrastructureMode == InfrastructureMode.Infrastructure)
                        {
                            webservices.location.Signal signal = new Signal();
                            signal.bssid = availableAPs[i].PhysicalAddress.ToString();
                            signal.ssid = availableAPs[i].Name;
                            signal.rssi = availableAPs[i].SignalStrength.Decibels;
                            signalVector.Add(signal);
                        }
                    }
                    // sort the list according to signal strength
                    if (signalVector.Count >= 2)
                    {
                        signalVector.Sort(delegate(Signal s1, Signal s2)
                        {
                            return -s1.rssi.CompareTo(s2.rssi);
                        });
                    }

                    if (signalVector.Count < 3)
                    {
                        numberOfAPs = signalVector.Count;
                    }
                    return signalVector.Take(numberOfAPs).ToArray();
                }
                return null;
            }
        }

        public bool IsConnected
        {
            get
            {
                if (nic != null)
                {
                    if (nic.AssociatedAccessPoint != string.Empty)
                    {
                        return true;
                    }
                }
                return false;
            }
        }

        public WirelessZeroConfigNetworkInterface Interface
        {
            get { return nic; }
        }
    }
}
