﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Services;
using System.Data.SqlClient;
using DDay.iCal;
using DDay.iCal.Components;

/// <summary>
/// Summary description for AnnouncementService
/// </summary>
[WebService(Namespace = "http://msis.myvnc.com/webservices/")]
[WebServiceBinding(ConformsTo = WsiProfiles.BasicProfile1_1)]
// To allow this Web Service to be called from script, using ASP.NET AJAX, uncomment the following line. 
// [System.Web.Script.Services.ScriptService]
public class AnnouncementService : System.Web.Services.WebService {

    // ADRESS TO CALENDAR SERVICE                           
    string icalUrl = "http://ntnu.1024.no/"; 


    public AnnouncementService () {

        //Uncomment the following line if using designed components 
        //InitializeComponent(); 
    }

    /// <summary>
    /// This method will return news/announcement for the user specified courses. 
    /// </summary>
    /// <param name="username">Username</param>
    /// <param name="pCourses">Specified news for courses ('general' gives global news)</param>
    /// <param name="timespanDays">Timespan in amount of days to include news from</param>
    /// <returns>Array of announcements/news</returns>
    [WebMethod]
    public Announcement[] GetAnnouncements(string username, string[] desiredCourses, int timespanDays) {
        
        // Prepare the list the announcement will be stored in
        List<Announcement> announcements = new List<Announcement>();
        
        // Make a list containing the user selection of courses and type of announcements the user has selected
        List<string> userCourseSelection = desiredCourses.ToList();
       

        // The first section handles general news (collected from the DB)
        if (userCourseSelection.Contains("general") || userCourseSelection.Contains("all"))
        {
            try
            {
                SQLConn.Open();
                
                // SQL Query for retrieving announcements that are intended for the username given in parameter
                string sqlQuery =
                       "SELECT Announcements.* FROM Announcements " +
                       "INNER JOIN Announcements_Users ON Announcements.id = Announcements_Users.announcement_id " +
                       "INNER JOIN Users ON Announcements_Users.user_id = Users.id " +
                       "WHERE Users.username = '" + username + "' " +
                       "ORDER BY Announcements.date DESC";

                // Prepare command and execute
                SqlCommand cmd = SQLConn.ExecuteCommand(sqlQuery);
                SqlDataReader reader = cmd.ExecuteReader();

                // For each record
                while (reader.Read())
                {
                    // Generate a new announcement and retrieve data from SQL results
                    Announcement a = new Announcement();
                    a.Category = (Announcement.AnnouncementCategory)Int32.Parse(reader["category"].ToString());
                    a.Title = (string)reader["title"];
                    a.Text = (string)reader["text"];
                    a.Course = null;
                    a.DateAdded = Convert.ToDateTime(reader["date"].ToString());
                    announcements.Add(a);
                }
                // Close reader and SQL Connection
                reader.Close();
                SQLConn.Close();
            }
            catch (Exception e)
            {
                // Writelog with error
                Logger.WriteLog(e.Message, false);
            }
        }

        // Retrieve the users courses from service
        Course[] userCourses = GetCoursesFromIcal(username, true);

        // This section handles specific courses in the user selection and tries to ascertain if they have RSS feeds.
        // If the courses have a RSS feed they will be downloaded
        foreach (Course course in userCourses)
        {
            // Foreach course the user has, we check if the parameter containing user courses inclnuded the course code, 
            // to specify if loop will proceed to get the course RSS feed contents
            if (userCourseSelection.Contains("all") || ContainsCourse(userCourseSelection, course.Code))
            {
                // Get the course feed URL
                string feedurl = GetCourseFeedURL(course.Code);

                // We won't bother to continue if course feed url is null
                if (feedurl != null)
                {

                    // Create a new RssReader object with the feed url
                    RssReader r = new RssReader(feedurl);


                    // Download the feed
                    r.GetFeed();

                    // Loop through each feed item and create new announcement objects, then add them to the list of announcements to be delievered 
                    foreach (Node item in r.Items)
                    {
                        Announcement a = new Announcement();

                        // We assume at this time that all rss feed items fall under the General category
                        a.Category = Announcement.AnnouncementCategory.General;
                        a.Title = item.Title;
                        a.Text = item.Description;
                        a.DateAdded = item.PubDate;
                        a.Course = course.Code;

                        announcements.Add(a);
                    }
                }
            }
        }

        // If the announcement list contains more than one item, we need to sort it
        if (announcements.Count > 0)
        {
            announcements.Sort();
            announcements.Reverse();

            // Check the condition that revolves around age of announcements, we will only return items newer than the set condition included in the parameters
            if (timespanDays > 0 || timespanDays == -1)
            {
                int days = timespanDays;

                if (timespanDays == -1)
                {
                    days = ComputeSemesterStart();
                }

                // Remove all elements that are older than the timespan of days set
                announcements.RemoveAll(delegate(Announcement a) { return (DateTime.Now - a.DateAdded).Days > days; });

            }
        }

        // If announcement count is 0, return null, otherwise we return a array version of the list
        return (announcements.Count > 0) ? announcements.ToArray() : null;
    }

    [WebMethod]
    public Course[] GetCoursesFromIcal(string username, bool returnOnlyCoursesWithFeeds)
    {
        // Basic object instances for method
        List<Course> courses = new List<Course>();
        string calUsername = "";
        
        int year = DateTime.Now.Year;
        
        // Set the correct semester string needed in the service url based on todays date
        string semester = DateTime.Now.Month > 7 ? "fall" : "spring";

        // We need the start and end of the period that we will return events from
        DateTime startDate = DateTime.Now.Month > 7 ? DateTime.Parse(year.ToString() + "-08-01") : DateTime.Parse(year.ToString() + "-01-01");
        DateTime endDate = DateTime.Now.Month > 7 ? DateTime.Parse(year.ToString() + "-12-31") : DateTime.Parse(year.ToString() + "-07-31");

        // End method if unable to connect to DB
        if (!SQLConn.Open())
        {
            return null;
        }

        // Prepare a SQL statement to retrieve calendar service username from DB
        SqlCommand com = SQLConn.ExecuteCommand("SELECT calendarusername FROM Users WHERE username = @UserName");
        com.Parameters.AddWithValue("UserName", username);

        // Execute the statement and retrieve the first result
        object result = com.ExecuteScalar();

        // Set the username if the result was not null / else return
        if (result != null)
            calUsername = result as string;
        else
            return null;
    
        // Also return if username was empty
        if (string.IsNullOrEmpty(calUsername))
            return null;

        // Construct the complete URL to the ical: http://urladress/year/semester/username
        icalUrl += string.Format("{0}/{1}/{2}/ical/", year.ToString(), semester, calUsername);

        // Create new iCalendar object and try to retrieve from specified URL adress
        iCalendar ical = iCalendar.LoadFromUri(new Uri(icalUrl));

        // Return null if unable to correctly create object
        if (ical == null)
            return null;

        // Evaluate events and check that they are within the timeframes set above. If not, they are discarded
        ical.Evaluate(startDate, endDate);

        // Loop through events
        foreach (Event evt in ical.Events)
        {
            bool newcourse = true;

            // Loop through the courses that are already found and check if this is a previously unfound course
            foreach (Course crs in courses)
            {
                if (evt.Summary.Value.Contains(crs.Code))               
                {
                    newcourse = false;
                }
            }

            // If this course has not been previously found then we create a new course and append it to the list
            if (newcourse)
            {
                Course c = new Course();
                c.Code = evt.Summary;
                
                try
                {
                    // Try to append the course name
                    // The format in the description field is:
                    //      TypeName - CourseName (CourseCode)
                    //          Eg. Forelesning - Objektorientert systemutvikling (IT2103)
                    // By using IndexOf/LastIndexOf searching for the first '-' and last '(' it should retrieve the correct values even if CourseName contains '-' or '('
                    c.Description = evt.Description.Value.Substring((evt.Description.Value.IndexOf('-') + 2), (evt.Description.Value.LastIndexOf('(') - 1) - (evt.Description.Value.IndexOf('-') + 2));
                }
                catch
                {
                    // An exception was thrown, some part of the code above failed. We leave the course name empty
                }

                courses.Add(c);

            }
        }

        // return an arrayed version of the list, or null if the list count is 0
        return courses.ToArray();
    }

    /// <summary>
    /// Query the database to see if a given course code has a know RSS feed URL
    /// </summary>
    /// <param name="course">Course code to check</param>
    /// <returns>The URL adress to the RSS feed</returns>
    private string GetCourseFeedURL(string course)
    {
        // Return null if not able to open SQL connection
        if (!SQLConn.Open())
        {
            return null;
        }

        // Prepare SQL command
        // Check the course table for a coursecode INCLUDING the parameter course. The reason for using LIKE @courseCode and adding %coursCode%
        // below is that sometimes there is a difference in how the coursecode is written, eg. TDT4100 or TDT4100-1. This should be covered with 
        // the sql statement below
        SqlCommand com = SQLConn.ExecuteCommand("SELECT rssfeed FROM Courses WHERE code LIKE @courseCode");
        com.Parameters.AddWithValue("courseCode", "%" + course + "%");

        object result = com.ExecuteScalar();

        // If the SQL query returns a result, we return it as a string
        if (com != null)
        {
            return result as string;
        }

        // If we reach the end of this method, the course doesn't have a rss feed
        return null;
    }

    /// <summary>
    /// Iterate a course list to check if a given coursecode is in the list
    /// </summary>
    /// <param name="courses">A list of courses</param>
    /// <param name="course">A specific course</param>
    /// <returns>True if the course is in the list, false otherwise</returns>
    private bool ContainsCourse(List<string> courses, string course)
    {
        if (courses == null)
        {
            return false;
        }

        foreach (string item in courses)
        {
            if (item.Equals(course)) return true;
        }
        return false;
    }

    /// <summary>
    /// Compute the timespan of days since the semester started
    /// </summary>
    /// <returns>Timespan of days since semester started</returns>
    private int ComputeSemesterStart()
    {
        DateTime date = DateTime.Now;
        int year = date.Year;
        int month = date.Month;
        int semstartMonth = 0;

        if (month > 7)
        {
            semstartMonth = 8;
        }
        else
        {
            semstartMonth = 1;
        }

        DateTime semesterStart = DateTime.Parse(year + "-" + semstartMonth + "-1");

        return (date - semesterStart).Days;

    }

    // This method is deprecated
    /*
    [WebMethod]
    public Course[] GetCourses(string username, bool returnOnlyCoursesWithFeeds)
    {
        List<Course> courses = new List<Course>();

        if (!SQLConn.Open())
        {
            return null;
        }
        
        SqlCommand com = SQLConn.ExecuteCommand("SELECT courses FROM Users WHERE username='" + username + "'");

        if (com != null)
        {
            using (SqlDataReader reader = com.ExecuteReader())
            {
                if (reader.HasRows && reader.Read())
                {
                    if (!string.IsNullOrEmpty(reader["courses"].ToString()))
                    {
                        string[] tmpCourses = (reader["courses"] as string).Split('|');

                        foreach (string course in tmpCourses)
                        {
                            Course c = new Course();
                            c.Code = course.Split('#')[0];
                            c.Description = course.Split('#')[1];
                            c.OutsideStudyProgram = Convert.ToBoolean(course.Split('#')[2]);
                            courses.Add(c);
                        }
                    }
                }
            }
        }

        if (courses.Count > 0)
        {
            if (returnOnlyCoursesWithFeeds)
            {
                string q = "SELECT * FROM Courses WHERE (code = '";

                for (int i = 0; i < courses.Count; i++)
                {
                    q += courses[i].Code + "'";

                    if ((i + 1) < courses.Count)
                    {
                        q += " OR code = '";
                    }
                }

                q += ") AND rssfeed IS NOT NULL";

                courses.Clear();

                SqlCommand cmd = SQLConn.ExecuteCommand(q);

                if (cmd != null)
                {
                    using (SqlDataReader reader = cmd.ExecuteReader())
                    {
                        while (reader.Read())
                        {
                            Course crs = new Course();
                            crs.Code = reader["code"].ToString();
                            crs.Description = reader["description"].ToString();
                            
                            courses.Add(crs);

                        }
                    }
                }

            }

        }


        return (courses.Count > 0) ? courses.ToArray() : null;
    }
 */
}

