﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;

/// <summary>
/// Summary description for Appointment
/// </summary>
public class Appointment
{
    public enum Type { Lecture, Exercise, Exam, Meeting, Other};
    Type appointmentType;
    DayOfWeek day;
    int startHour; // 0-23
    int startMinute; // 0-60
    int endHour; // 0-23
    int endMinute; // 0-60
    string title;
    string room;
    string person;
    int[] weeks;
    string weeksString;
    DateTime date;

    public Appointment()
    {
        // default constructor
    }

    /// <summary>
    /// Constructor.
    /// </summary>
    /// <param name="type"></param>
    /// <param name="day"></param>
    /// <param name="start">Must be specified on the format HH:MM</param>
    /// <param name="end">Must be specified on the format HH:MM</param>
    /// <param name="text"></param>
	public Appointment(Type type, DayOfWeek day, string start, string end, string title, string room)
	{
        this.appointmentType = type;
        this.day = day;
        this.startHour = int.Parse(start.Split(':')[0]);
        this.startMinute = int.Parse(start.Split(':')[1]);
        this.endHour = int.Parse(end.Split(':')[0]);
        this.endMinute = int.Parse(end.Split(':')[1]);
        this.title = title;
        this.room = room;
	}

    public Type AppointmentType
    {
        get { return appointmentType; }
        set { appointmentType = value; }
    }
    public DayOfWeek Day
    {
        get { return day; }
        set { day = value; }
    }
    public int StartHour
    {
        get { return startHour; }
        set { startHour = value; }
    }
    public int StartMinute
    {
        get { return startMinute; }
        set { startMinute = value; }
    }
    public int EndHour
    {
        get { return endHour; }
        set { endHour = value; }
    }
    public int EndMinute
    {
        get { return endMinute; }
        set { endMinute = value; }
    }
    /// <summary>
    /// Returns the duration of the appointment in minutes
    /// </summary>
    public int Duration
    {
        set { }
        get { return ((endHour - startHour) * 60) + (endMinute - startMinute); }
    }
    public string Title
    {
        get { return title; }
        set { title = value; }
    }
    public string Room
    {
        get { return room; }
        set { room = value; }
    }
    public string Person
    {
        get { return person; }
        set { person = value; }
    }
    public int[] Weeks
    {
        get { return weeks; }
        set { weeks = value; }
    }
    public string WeeksString
    {
        get { return weeksString; }
        set { weeksString = value; }
    }
    public DateTime Date
    {
        get { return date; }
        set { date = value; }
    }

    public override string ToString()
    {
        return title + " @ " + day + " " + startHour + ":" + startMinute + "-" + endHour + ":" + endMinute + " in " + room;
    }
}
