﻿using System;
using System.Collections.Generic;
using System.Xml;
using System.Collections.ObjectModel;
using System.Linq;
using System.Web;
using System.Xml.Linq;

/// <summary>
/// RssReader object retrieves RSS feed items from a URL
/// </summary>
public class RssReader
{
    private string adress;
    private Collection<Node> rssitems;

    /// <summary>
    /// Creates a new instance of RssReader with a given adress
    /// </summary>
    /// <param name="adress">URL adress for feed</param>
	public RssReader(string adress)
	{
        this.adress = adress;
        rssitems = new Collection<Node>();
	}

    /// <summary>
    /// Retrieve items from RSS feed
    /// </summary>
    public void GetFeed()
    {
        if (string.IsNullOrEmpty(adress)) return;

        try
        {
            using (XmlReader reader = XmlReader.Create(adress))
            {
                XDocument doc = XDocument.Load(reader);
                ParseRSSItems(doc);
            }
        }
        catch
        {
            // Error loading stream
            // Return without parsing items, as there are non to parse
        }

    }

    private void ParseRSSItems(XDocument doc)
    {
        // LINQ statement that retrieves all xml elements with the tag: item
        var items = from item in doc.Descendants("item")
                    select item;

        // loops through the result of the LINQ statement and creates new Nodes
        foreach (var item in items) 
        {
            Node n = new Node();
            n.Title = item.Element("title") == null ? null : item.Element("title").Value;
            n.Link = item.Element("link") == null ? null : item.Element("link").Value;
            n.PubDate = item.Element("pubDate") == null ? DateTime.MinValue : Convert.ToDateTime(item.Element("pubDate").Value);
            n.Description = item.Element("description") == null ? null : item.Element("description").Value;
            
            rssitems.Add(n);
        }
    }

    /// <summary>
    /// Returns a collection with the retrieved RSS feed items (remember to use GetFeed() first)
    /// </summary>
    public Collection<Node> Items
    {
        get { return rssitems; }
    }

}

public class Node
{
    public string Title { set; get; }
    public DateTime PubDate { set; get; }
    public string Description { set; get; }
    public string Link { set; get; }
}
