﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Services;
using System.Data.SqlClient;

/// <summary>
/// This web service is responsible for account management and login authentication functionality.
/// </summary>
[WebService(Namespace = "http://msis.myvnc.com/webservices/")]
[WebServiceBinding(ConformsTo = WsiProfiles.BasicProfile1_1)]
// To allow this Web Service to be called from script, using ASP.NET AJAX, uncomment the following line. 
// [System.Web.Script.Services.ScriptService]
public class UserService : System.Web.Services.WebService {

    public UserService () {

        //Uncomment the following line if using designed components 
        //InitializeComponent(); 
    }

    /// <summary>
    /// The method accepts a username and password and performs login authentication against the database of 
    /// known users. If the user is successfully authenticated, a UserProfile object is returned to the 
    /// requester containing the details about the user.
    /// </summary>
    /// <param name="username">The username</param>
    /// <param name="password">The SHA1-encrypted hash sum of the password</param>
    /// <returns>A UserProfile object for the user</returns>
    [WebMethod]
    public UserProfile Login(string username, string password)
    {
        UserProfile user = new UserProfile();

        // Terminate operation and return if unable to connect to DB
        if (!SQLConn.Open())
        {
            return null;
        }

        // Create SqlCommand and prepare statement
        using (SqlCommand cmd = SQLConn.ExecuteCommand("SELECT * FROM Users WHERE username='" + username + "' AND password='" + password + "'"))
        {
            // Execute SQL query
            using (SqlDataReader result = cmd.ExecuteReader())
            {
                // If execution gives a result:
                if (result.Read())
                {
                    // Create userprofile
                    user.IsAuthenticated = true;
                    user.Id = Int32.Parse(result["id"].ToString());
                    user.Name = (string)result["name"];
                    user.Username = (string)result["username"];
                    user.Email = (string)result["email"];
                    user.CalendarServiceUsername = (string)result["calendarusername"];
                }
                result.Close();
            }
        }

        // Close up and return the UserProfile object
        SQLConn.Close();
        return user;
    }

    /// <summary>
    /// The method should be used to modify the profile of an existing user.
    /// The method accepts a UserProfile object and optionally a new password (encrypted using the 
    /// SHA1 algorithm) for the account. If the password should remain unchanged, just pass an empty string.
    /// </summary>
    /// <param name="user">The user object</param>
    /// <param name="newPassword">The SHA1-encrypted hash sum of the new password (leave empty to keep current password)</param>
    /// <returns>'True' if update was successfull, 'False' otherwise</returns>
    [WebMethod]
    public bool UpdateUser(UserProfile user, string newPassword)
    {
        // Terminate if unable to open SQL Connection
        if (!SQLConn.Open())
        {
            return false;
        }

        // Create SqlCommand object and prepare the statement according to two different scenarios:
        //      a) The user wants to change password 
        //      b) The user do not want to change the password.
        SqlCommand cmd;
        if (newPassword.Length > 0)
        {
            cmd = SQLConn.ExecuteCommand("UPDATE Users SET name= @name, email = @email, username = @username, password = @password, calendarusername = @cusername WHERE id=" + user.Id);
            cmd.Parameters.AddWithValue("name", user.Name);
            cmd.Parameters.AddWithValue("email", user.Email);
            cmd.Parameters.AddWithValue("username", user.Username);
            cmd.Parameters.AddWithValue("password", newPassword);
            cmd.Parameters.AddWithValue("cusername", user.CalendarServiceUsername);
        }
        else
        {
            cmd = SQLConn.ExecuteCommand("UPDATE Users SET name = @name, email = @email, username = @username, calendarusername = @cusername WHERE id=" + user.Id);
            cmd.Parameters.AddWithValue("name", user.Name);
            cmd.Parameters.AddWithValue("email", user.Email);
            cmd.Parameters.AddWithValue("username", user.Username);
            cmd.Parameters.AddWithValue("cusername", user.CalendarServiceUsername);
        }

        // Execute the Sql statement
        using(cmd)
        {
            if (cmd.ExecuteNonQuery() > 0)
            {
                // Execution was successfull. Return 'true' to indicate to sender that the operation went well.
                return true;
            }
        }

        // Execution was unsuccessfull. Return 'false' to indicate this failure to the sender.
        SQLConn.Close();
        return false;
    }

    /// <summary>
    /// This method is used to create new users for the MSIS service. The method accepts a UserProfile object and a SHA1 encrypted password
    /// </summary>
    /// <param name="user">The user object</param>
    /// <param name="password">The SHA1-encrypted hash sum of the new password</param>
    /// <returns>'True' if update was successfull, 'False' otherwise</returns>
    [WebMethod]
    public bool NewUser(UserProfile user, string password)
    {
        // No need to bother continuing if user object is null or password is blank/null
        if (user == null || string.IsNullOrEmpty(password))
        {
            return false;
        }

        // Terminate if unable to open SQL Connection
        if (!SQLConn.Open())
        {
            return false;
        }

        // Create SQLCommand object and prepare statement
        SqlCommand cmd;
        cmd = SQLConn.ExecuteCommand("INSERT INTO Users (name, email, username, password, calendarusername) VALUES (@name, @email, @username, @password, @calendarusername)");
        cmd.Parameters.AddWithValue("name", user.Name);
        cmd.Parameters.AddWithValue("email", user.Email);
        cmd.Parameters.AddWithValue("username", user.Username);
        cmd.Parameters.AddWithValue("password", password);
        cmd.Parameters.AddWithValue("calendarusername", user.CalendarServiceUsername);

        // Execute command
        using (cmd)
        {
            if (cmd.ExecuteNonQuery() > 0)
            {
                // Execution completed successfully, resulting in a new row in DB. Returns 'true' to indicate this success.
                return true;
            }
        }

        // Terminate and return false (indicates failure) at this stage (successfull path above)
        SQLConn.Close();
        return false;
    }
    
}

